def test_demo():
    """
    Assign a value to `x` after an assert
    Testsuite will want to ensure that we print `x = 1`, which was the value at
    the time of the assert
    """
    x = 1
    assert x == 2
    x = 2



import logging
import multiprocessing
import multiprocessing.connection as connection
import os
import select
import sys
import unittest
from collections.abc import Sequence

from nose2 import events, loader, result, runner, session, util

log = logging.getLogger(__name__)


class MultiProcess(events.Plugin):
    configSection = "multiprocess"

    def __init__(self):
        self.addArgument(
            self.setProcs,
            "N",
            "processes",
            "Number of processes used to run tests (0 = auto)",
        )
        self.testRunTimeout = self.config.as_float("test-run-timeout", 60.0)
        self._procs = self.config.as_int("processes", 0)
        self.setAddress(self.config.as_str("bind_address", None))

        self.cases = {}

    @property
    def procs(self):
        """Get the appropriate number of procs for self.procs if self._procs is
        0."""

        if self._procs == 0:
            try:
                self._procs = multiprocessing.cpu_count()
            except NotImplementedError:
                self._procs = 1
        return self._procs

    @procs.setter
    def procs(self, value):
        """Setter for procs property"""
        if value < 0:
            raise ValueError("Number of processes cannot be less than 0")
        self._procs = value

    def setProcs(self, num):
        self.procs = int(num[0])  # FIXME merge n fix
        self.register()

    def setAddress(self, address):
        if address is None or address.strip() == "":
            address = []
        else:
            address = [x.strip() for x in address.split(":")[:2]]

        # Background:  On Windows, select.select only works on sockets.  So the
        # ability to select a bindable address and optionally port for the mp
        # plugin was added.  Pipes should support a form of select, but this
        # would require using pywin32.  There are alternatives but all have
        # some kind of downside.  An alternative might be creating a connection
        # like object using a shared queue for incomings events.
        self.bind_host = None
        self.bind_port = 0

        if sys.platform == "win32" or address:
            self.bind_host = "127.116.157.163"
            if address and address[0]:
                self.bind_host = address[0]

            self.bind_port = 0
            if len(address) >= 2:
                self.bind_port = int(address[1])

    def pluginsLoaded(self, event):
        self.addMethods("registerInSubprocess", "startSubprocess", "stopSubprocess")

    def startTestRun(self, event):
        event.executeTests = self._runmp

    def beforeInteraction(self, event):
        # prevent interactive plugins from running
        event.handled = True
        return False

    def _runmp(self, test, result):
        # flatten technically modifies a hash of test cases, let's
        # only run it once per run.
        flat = list(self._flatten(test))

        # do not send import failures to the subprocesses, which will mangle them
        # but 'run' them in the main process.
        failed_import_id = "nose2.loader.LoadTestsFailure"
        result_ = self.session.testResult
        for testid in flat:
            if testid.startswith(failed_import_id):
                self.cases[testid].run(result_)

        # XXX Process-Handling: The length of the filtered list needs to be
        # known for _startProcs, until this can be cleaned up.  This
        # wasn't the best way to deal with too few tests
        flat = [x for x in flat if not x.startswith(failed_import_id)]
        procs = self._startProcs(len(flat))

        # send one initial task to each process
        for _proc, conn in procs:
            if not flat:
                break
            caseid = flat.pop(0)
            # NOTE: it throws errors on broken pipes and bad serialization
            conn.send(caseid)

        rdrs = [conn for proc, conn in procs if proc.is_alive()]
        while flat or rdrs:
            ready, _, _ = select.select(rdrs, [], [], self.testRunTimeout)
            for conn in ready:
                # XXX Process-Handling: If we get an EOFError on receive the
                # process finished= or we lost the process and the test it was
                # working on.  Also do we rebuild the process?
                try:
                    remote_events = conn.recv()
                except EOFError:
                    # probably dead/12
                    log.warning("Subprocess connection closed unexpectedly")
                    continue

                # If remote_events is None, the process exited normally,
                # which should mean that we didn't any more tests for it.
                if remote_events is None:
                    log.debug("Conn closed %s", conn)
                    rdrs.remove(conn)
                    continue

                # replay events
                testid, events = remote_events
                log.debug("Received results for %s", testid)
                for hook, event in events:
                    log.debug("Received %s(%s)", hook, event)
                    self._localize(event)
                    getattr(self.session.hooks, hook)(event)

                # Send the next test_id
                # NOTE: send throws errors on broken pipes and bad serialization
                if not flat:
                    # If there are no more, send None - it's the 'done' flag
                    conn.send(None)
                    continue
                caseid = flat.pop(0)
                conn.send(caseid)

        for _, conn in procs:
            conn.close()

        # ensure we wait until all processes are done before
        # exiting, to allow plugins running there to finalize
        for proc, _ in procs:
            proc.join()

    def _prepConns(self):
        """
        If the ``bind_host`` is not ``None``, return:
            (multiprocessing.connection.Listener, (address, port, authkey))
        else:
            (parent_connection, child_connection)

        For the former case: ``accept`` must be called on the listener. In order
        to get a ``Connection`` object for the socket.
        """
        if self.bind_host is not None:
            # prevent "accidental" wire crossing
            authkey = os.urandom(20)
            address = (self.bind_host, self.bind_port)
            listener = connection.Listener(address, authkey=authkey)
            return (listener, listener.address + (authkey,))
        else:
            return multiprocessing.Pipe()

    def _acceptConns(self, parent_conn):
        """
        When listener is is a :class:`connection.Listener` instance: accept the next
        incoming connection.  However, a timeout mechanism is needed.  Since
        this functionality was added to support mp over inet sockets, this method
        assumes a Socket-based listen, and will accept the private _socket
        member to get a low_level socket to do a select on.
        """
        if isinstance(parent_conn, connection.Listener):
            # ick private interface
            rdrs = [parent_conn._listener._socket]
            readable, _, _ = select.select(rdrs, [], [], self.testRunTimeout)
            if readable:
                return parent_conn.accept()
            else:
                raise RuntimeError("MP: Socket Connection Failed")
        else:
            return parent_conn

    def _startProcs(self, test_count):
        # Create session export
        session_export = self._exportSession()
        procs = []
        count = min(test_count, self.procs)
        log.debug("Creating %i worker processes", count)
        for _ in range(0, count):
            parent_conn, child_conn = self._prepConns()
            proc = multiprocessing.Process(
                target=procserver, args=(session_export, child_conn)
            )
            proc.daemon = True
            proc.start()
            parent_conn = self._acceptConns(parent_conn)
            procs.append((proc, parent_conn))
        return procs

    def _flatten(self, suite):
        """
        Flatten test-suite into list of IDs, AND record all test case
        into self.cases

        CAVEAT:  Due to current limitation of the MP plugin, examine the suite
                 tests to find out if they have class or module fixtures and
                 group them that way into name of test classes or module.
                 This is aid in their dispatch.
        """
        log.debug("Flattening test into list of IDs")
        mods = {}
        classes = {}
        stack = [suite]
        while stack:
            suite = stack.pop()
            for test in suite:
                if isinstance(test, unittest.TestSuite):
                    stack.append(test)
                else:
                    testid = util.test_name(test)
                    self.cases[testid] = test
                    if util.has_module_fixtures(test):
                        mods.setdefault(test.__class__.__module__, []).append(testid)
                    elif util.has_class_fixtures(test):
                        # testclasses support
                        if test.__class__.__name__ == "_MethodTestCase":
                            test = test.obj
                        classes.setdefault(
                            "%s.%s"
                            % (test.__class__.__module__, test.__class__.__name__),
                            [],
                        ).append(testid)
                    else:
                        yield testid

        yield from sorted(classes.keys())
        yield from sorted(mods.keys())

    def _localize(self, event):
        # XXX set loader, case, result etc to local ones, if present in event
        # (event case will be just the id)
        # (traceback in exc_info if any won't be real!)
        if hasattr(event, "result"):
            event.result = self.session.testResult
        if hasattr(event, "loader"):
            event.loader = self.session.testLoader
        if hasattr(event, "runner"):
            event.runner = self.session.testRunner
        if hasattr(event, "test") and isinstance(event.test, str):
            # remote event.case is the test id
            try:
                event.test = self.cases[event.test]
            except KeyError:
                # this happens when _flatten augments the test suite
                # due to a class or module fixture being present
                event.test = self.session.testLoader.loadTestsFromName(
                    event.test
                )._tests[0]
            # subtest support
            if "subtest" in event.metadata:
                message, params = event.metadata.pop("subtest")
                # XXX the sentinel value does not survive the pickle
                # round-trip and must be reset by hand
                if type(message) == type(object()):  # noqa: E721
                    message = unittest.case._subtest_msg_sentinel
                event.test = unittest.case._SubTest(event.test, message, params)

    def _exportSession(self):
        """
        Generate the session information passed to work process.

        CAVEAT: The entire contents of which *MUST* be pickleable
        and safe to use in the subprocess.

        This probably includes:
        * No argparse namespaces/named-tuples
        * No plugin instances
        * No hokes
        :return:
        """
        export = {
            "config": self.session.config,
            "verbosity": self.session.verbosity,
            "startDir": self.session.startDir,
            "topLevelDir": self.session.topLevelDir,
            "logLevel": self.session.logLevel,
            "pluginClasses": [],
        }
        event = RegisterInSubprocessEvent()
        # fire registerInSubprocess on plugins -- add those plugin classes
        # CAVEAT: classes must be pickleable!
        self.session.hooks.registerInSubprocess(event)
        export["pluginClasses"].extend(event.pluginClasses)
        return export


def procserver(session_export, conn):
    # init logging system
    rlog = multiprocessing.log_to_stderr()
    rlog.setLevel(session_export["logLevel"])

    # make a real session from the "session" we got
    ssn = import_session(rlog, session_export)

    if isinstance(conn, Sequence):
        conn = connection.Client(conn[:2], authkey=conn[2])

    event = SubprocessEvent(
        ssn.testLoader, ssn.testResult, ssn.testRunner, ssn.plugins, conn
    )
    res = ssn.hooks.startSubprocess(event)
    if event.handled and not res:
        conn.send(None)
        conn.close()
        ssn.hooks.stopSubprocess(event)
        return
    # receive and run tests
    executor = event.executeTests
    for testid in gentests(conn):
        if testid is None:
            break
        # XXX to handle weird cases like layers, need to
        # deal with the case that testid is something other
        # than a simple string.
        test = event.loader.loadTestsFromName(testid)
        # XXX If there a need to protect the loop? try/except?
        rlog.debug("Execute test %s (%s)", testid, test)
        executor(test, event.result)
        events = list(ssn.hooks.flush())
        try:
            conn.send((testid, events))
            rlog.debug("Log for %s returned", testid)
        except Exception:
            rlog.exception(f"Fail sending event {testid}: {events}")
            # Send empty event list to unblock the conn.recv on main process.
            conn.send((testid, []))
    conn.send(None)
    conn.close()
    ssn.hooks.stopSubprocess(event)


def import_session(rlog, session_export):
    ssn = session.Session()
    ssn.config = session_export["config"]
    ssn.hooks = RecordingPluginInterface()
    ssn.verbosity = session_export["verbosity"]
    ssn.startDir = session_export["startDir"]
    ssn.topLevelDir = session_export["topLevelDir"]
    ssn.prepareSysPath()
    loader_ = loader.PluggableTestLoader(ssn)
    ssn.testLoader = loader_
    result_ = result.PluggableTestResult(ssn)
    ssn.testResult = result_
    runner_ = runner.PluggableTestRunner(ssn)  # needed??
    ssn.testRunner = runner_
    # load and register plugins, forcing multiprocess to the end
    ssn.plugins = [
        plugin(session=ssn)
        for plugin in session_export["pluginClasses"]
        if plugin is not MultiProcess
    ]
    rlog.debug("Plugins loaded: %s", ssn.plugins)

    for plugin in ssn.plugins:
        plugin.register()
        rlog.debug("Registered %s in subprocess", plugin)

    # instantiating the plugin will register it.
    ssn.plugins.append(MultiProcess(session=ssn))
    rlog.debug("Registered %s in subprocess", MultiProcess)
    ssn.plugins[-1].pluginsLoaded(events.PluginsLoadedEvent(ssn.plugins))
    return ssn


# test generator
def gentests(conn):
    while True:
        try:
            testid = conn.recv()
            if testid is None:
                return
            yield testid
        except EOFError:
            return


# custom event classes
class SubprocessEvent(events.Event):

    """Event fired at start and end of subprocess execution.

    .. attribute :: loader

       Test loader instance

    .. attribute :: result

       Test result

    .. attribute :: plugins

       List of plugins loaded in the subprocess.

    .. attribute :: connection

       The :class:`multiprocessing.Connection` instance that the
       subprocess uses for communication with the main process.

    .. attribute :: executeTests

       Callable that will be used to execute tests.  Plugins may set
       this attribute to wrap or otherwise change test execution. The
       callable must match the signature::

         def execute(suite, result):
             ...

    """

    def __init__(self, loader, result, runner, plugins, connection, **metadata):
        self.loader = loader
        self.result = result
        self.runner = runner
        self.plugins = plugins
        self.connection = connection
        self.executeTests = lambda test, result: test(result)
        super().__init__(**metadata)


class RegisterInSubprocessEvent(events.Event):

    """Event fired to notify plugins that multiprocess testing will occur

    .. attribute :: pluginClasses

       Add a plugin class to this list to cause the plugin to be
       instantiated in each test-running subprocess. The most common
       thing to do, for plugins that need to run in subprocesses, is::

         def registerInSubprocess(self, event):
             event.pluginClasses.append(self.__class__)

    """

    def __init__(self, **metadata):
        self.pluginClasses = []
        super().__init__(**metadata)


# custom hook system that records calls and events
class RecordingHook(events.Hook):
    def __init__(self, method, interface):
        super().__init__(method)
        self.interface = interface

    def __call__(self, event):
        res = super().__call__(event)
        self.interface.log(self.method, event)
        return res


class RecordingPluginInterface(events.PluginInterface):
    hookClass = RecordingHook
    noLogMethods = {
        "getTestCaseNames",
        "startSubprocess",
        "stopSubprocess",
        "registerInSubprocess",
        "moduleLoadedSuite",
        "getTestMethodNames",
    }

    def __init__(self):
        super().__init__()
        self.events = []

    def log(self, method, event):
        self.events.append((method, event))

    def flush(self):
        events = self.events[:]
        self.events = []
        return events

    def register(self, method, plugin):
        """Register a plugin for a method.

        :param method: A method name
        :param plugin: A plugin instance

        """
        self._hookForMethod(method).append(plugin)

    def __getattr__(self, attr):
        if attr.startswith("__"):
            raise AttributeError(f"No {attr} in {self}")
        return self._hookForMethod(attr)

    def _hookForMethod(self, method):
        # return recording hook for most hooks, normal hook for those
        # (like test loading and subprocess events) that we don't want
        # to send back to the main process.
        try:
            return self.hooks[method]
        except KeyError:
            if method in self.noLogMethods or method.startswith("loadTest"):
                hook = events.Hook(method)
            else:
                hook = self.hookClass(method, self)
        self.hooks[method] = hook
        return hook



import unittest


class L1:
    @classmethod
    def setUp(cls):
        print("L1 setUp")

    @classmethod
    def testSetUp(cls):
        print("L1 testSetUp")

    @classmethod
    def tearDown(cls):
        print("L1 tearDown")

    @classmethod
    def testTearDown(cls):
        print("L1 testTearDown")


class L2(L1):
    @classmethod
    def setUp(cls):
        print("L2 setUp")

    @classmethod
    def testSetUp(cls):
        print("L2 testSetUp")

    @classmethod
    def testTearDown(cls):
        print("L2 testTearDown")


# L1 tearDown should only run once
class T1(unittest.TestCase):
    layer = L2

    def test1(self):
        print("Run test1")

    def test2(self):
        print("Run test2")



"""
Egg-based discovery test loader.

This plugin implements nose2's automatic test module discovery inside Egg Files.
It looks for test modules in packages whose names start
with ``test``, then fires the :func:`loadTestsFromModule` hook for each
one to allow other plugins to load the actual tests.

It also fires :func:`handleFile` for every file that it sees, and
:func:`matchPath` for every Python module, to allow other plugins to
load tests from other kinds of files and to influence which modules
are examined for tests.

"""
import logging
import os

from nose2 import events
from nose2.plugins.loader import discovery

__unittest = True
log = logging.getLogger(__name__)

try:
    import pkg_resources
except ImportError:
    pkg_resources = None


class EggDiscoveryLoader(events.Plugin, discovery.Discoverer):
    """Loader plugin that can discover tests inside Egg Files"""

    alwaysOn = True
    configSection = "discovery"

    def registerInSubprocess(self, event):
        event.pluginClasses.append(self.__class__)

    def loadTestsFromName(self, event):
        """Load tests from module named by event.name"""
        return discovery.Discoverer.loadTestsFromName(self, event)

    def loadTestsFromNames(self, event):
        """Discover tests if no test names specified"""
        return discovery.Discoverer.loadTestsFromNames(self, event)

    def _checkIfPathIsOK(self, start_dir):
        if not os.path.exists(os.path.abspath(start_dir)):
            raise OSError("%s does not exist" % os.path.abspath(start_dir))

    def _find_tests_in_egg_dir(self, event, rel_path, dist):
        log.debug(
            "find in egg dir %s %s (%s)", dist.location, rel_path, dist.project_name
        )
        full_path = os.path.join(dist.location, rel_path)
        dir_handler = discovery.DirectoryHandler(self.session)
        yield from dir_handler.handle_dir(event, full_path, dist.location)
        if dir_handler.event_handled:
            return
        for path in dist.resource_listdir(rel_path):
            # on setuptools==38.2.5 , resource_listdir() can yield ""
            # if that happens, skip processing it to avoid infinite recursion
            if path == "":
                continue

            entry_path = os.path.join(rel_path, path)
            if dist.resource_isdir(entry_path):
                yield from self._find_tests_in_egg_dir(event, entry_path, dist)
            else:
                modname = os.path.splitext(entry_path)[0].replace(os.sep, ".")
                yield from self._find_tests_in_file(
                    event,
                    path,
                    os.path.join(dist.location, entry_path),
                    dist.location,
                    modname,
                )

    def _find_tests_in_dir(self, event, full_path, top_level):
        if os.path.exists(full_path):
            return
        elif pkg_resources and full_path.find(".egg") != -1:
            egg_path = full_path.split(".egg")[0] + ".egg"
            for dist in pkg_resources.find_distributions(egg_path):
                for modname in dist._get_metadata("top_level.txt"):
                    yield from self._find_tests_in_egg_dir(event, modname, dist)



import unittest


class TestFailAssert(unittest.TestCase):
    def test_failing_assert(self):
        x = True
        y = False
        # fmt: off
        # ruff: noqa
        assert x; assert y
        # fmt: on

    def test_failing_assert2(self):
        p = 1
        q = 0
        assert p
        assert q



from nose2.tests._common import FunctionalTestCase, support_file


class TestDoctestsPlugin(FunctionalTestCase):
    def test_simple(self):
        proc = self.runIn(
            "scenario/doctests",
            "-v",
            "--plugin=nose2.plugins.doctests",
            "--with-doctest",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 6 tests")
        self.assertTestRunOutputMatches(proc, stderr="Doctest: docs ... ok")
        self.assertTestRunOutputMatches(proc, stderr="Doctest: docs.rst ... ok")
        self.assertTestRunOutputMatches(proc, stderr="Doctest: docs.txt ... ok")
        self.assertTestRunOutputMatches(
            proc, stderr="Doctest: doctests_pkg1.docs1 ... ok"
        )
        self.assertTestRunOutputMatches(proc, stderr="Doctest: docs1.rst ... ok")
        self.assertTestRunOutputMatches(proc, stderr="Doctest: docs1.txt ... ok")
        self.assertEqual(proc.poll(), 0)

    def test_start_directory_inside_package(self):
        proc = self.runIn(
            "scenario/doctests/doctests_pkg1",
            "-v",
            "--plugin=nose2.plugins.doctests",
            "--with-doctest",
            "-t",
            support_file("scenario/doctests"),
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 3 tests")
        self.assertTestRunOutputMatches(
            proc, stderr="Doctest: doctests_pkg1.docs1 ... ok"
        )
        self.assertTestRunOutputMatches(proc, stderr="Doctest: docs1.rst ... ok")
        self.assertTestRunOutputMatches(proc, stderr="Doctest: docs1.txt ... ok")
        self.assertEqual(proc.poll(), 0)

    def test_project_directory_inside_package(self):
        proc = self.runIn(
            "scenario/doctests/doctests_pkg1",
            "-v",
            "--plugin=nose2.plugins.doctests",
            "--with-doctest",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 3 tests")
        self.assertTestRunOutputMatches(
            proc, stderr="Doctest: doctests_pkg1.docs1 ... ok"
        )
        self.assertTestRunOutputMatches(proc, stderr="Doctest: docs1.rst ... ok")
        self.assertTestRunOutputMatches(proc, stderr="Doctest: docs1.txt ... ok")
        self.assertEqual(proc.poll(), 0)



import unittest


class Layer1:
    layer_setup = 0
    test_setup = 0
    test_teardown = 0
    layer_teardown = 0
    tests_count = 0

    @classmethod
    def setUp(cls):
        if cls.layer_setup >= 1:
            raise Exception("layer_setup already ran")
        cls.layer_setup += 1

    @classmethod
    def testSetUp(cls):
        if cls.test_setup >= 2:
            raise Exception("test_setup already ran twice")
        cls.test_setup += 1

    @classmethod
    def testTearDown(cls):
        if cls.test_teardown >= 2:
            raise Exception("test_teardown already ran twice")
        cls.test_teardown += 1

    @classmethod
    def tearDown(cls):
        if cls.layer_teardown >= 1:
            raise Exception("layer_teardown already ran")
        cls.layer_teardown += 1


class TestSimple(unittest.TestCase):
    layer = Layer1

    def test_1(self):
        assert self.layer.layer_setup == 1
        assert self.layer.test_setup == self.layer.tests_count + 1
        assert self.layer.test_teardown == self.layer.tests_count
        assert self.layer.layer_teardown == 0
        self.layer.tests_count += 1

    def test_2(self):
        assert self.layer.layer_setup == 1
        assert self.layer.test_setup == self.layer.tests_count + 1
        assert self.layer.test_teardown == self.layer.tests_count
        assert self.layer.layer_teardown == 0
        self.layer.tests_count += 1



from nose2.tools import such

with such.A("system with complex setup") as it:

    @it.has_setup
    def setup():
        it.things = [1]

    @it.has_teardown
    def teardown():
        it.things = []

    @it.should("do something")
    def test():
        assert it.things
        it.assertEqual(len(it.things), 1)

    with it.having("an expensive fixture"):

        @it.has_setup  # noqa: F811
        def setup():  # noqa: F811
            it.things.append(2)

        @it.should("do more things")  # noqa: F811
        def test(case):  # noqa: F811
            case.assertEqual(it.things[-1], 2)


it.createTests(globals())



# This module contains some code copied from unittest2/loader.py and other
# code developed in reference to that module and others within unittest2.

# unittest2 is Copyright (c) 2001-2010 Python Software Foundation; All
# Rights Reserved. See: http://docs.python.org/license.html

import inspect
import logging
import os
import re
import sys
import traceback
import types

__unittest = True
IDENT_RE = re.compile(r"^[_a-zA-Z]\w*$", re.UNICODE)
VALID_MODULE_RE = re.compile(r"[_a-zA-Z]\w*\.py$", re.UNICODE)


def ln(label, char="-", width=70):
    """Draw a divider, with ``label`` in the middle.

    >>> ln('hello there')
    '---------------------------- hello there -----------------------------'

    ``width`` and divider ``char`` may be specified. Defaults are ``70`` and ``'-'``,
    respectively.

    """
    label_len = len(label) + 2
    chunk = (width - label_len) // 2
    out = f"{char * chunk} {label} {char * chunk}"
    pad = width - len(out)
    if pad > 0:
        out = out + (char * pad)
    return out


def valid_module_name(path):
    """Is ``path`` a valid module name?"""
    return VALID_MODULE_RE.search(path)


def name_from_path(path):
    """Translate ``path`` into module name

    Returns a two-element tuple:

    1. a dotted module name that can be used in an import statement
       (e.g., ``pkg1.test.test_things``)

    2. a full path to filesystem directory, which must be on ``sys.path``
       for the import to succeed.

    """
    # back up to find module root
    parts = []
    path = os.path.normpath(path)
    base = os.path.splitext(path)[0]
    candidate, top = os.path.split(base)
    parts.append(top)
    while candidate:
        if ispackage(candidate):
            candidate, top = os.path.split(candidate)
            parts.append(top)
        else:
            break
    return ".".join(reversed(parts)), candidate


def module_from_name(name):
    """Import module from ``name``"""
    __import__(name)
    return sys.modules[name]


def test_from_name(name, module):
    """Import test from ``name``"""
    pos = name.find(":")
    index = None
    if pos != -1:
        real_name, digits = name[:pos], name[pos + 1 :]
        try:
            index = int(digits)
        except ValueError:
            pass
        else:
            name = real_name
    parent, obj = object_from_name(name, module)
    return parent, obj, name, index


def object_from_name(name, module=None):
    """
    Given a dotted name, return the corresponding object.

    Getting the object can fail for two reason:

        - the object is a module that cannot be imported.
        - the object is a class or a function that does not exists.

    Since we cannot distinguish between these two cases, we assume we are in
    the first one. We expect the stacktrace is explicit enough for the user to
    understand the error.
    """
    import_error = None
    parts = name.split(".")
    if module is None:
        (module, import_error) = try_import_module_from_name(parts[:])
        parts = parts[1:]
    parent = None
    obj = module
    for part in parts:
        try:
            parent, obj = obj, getattr(obj, part)
        except AttributeError as e:
            if is_package_or_module(obj) and import_error:
                raise e from import_error[1]
            raise

    return parent, obj


def is_package_or_module(obj):
    if hasattr(obj, "__path__") or isinstance(obj, types.ModuleType):
        return True
    return False


def try_import_module_from_name(splitted_name):
    """
    Try to find the longest importable from the ``splitted_name``, and return
    the corresponding module, as well as the potential ``ImportError``
    exception that occurs when trying to import a longer name.

    For instance, if ``splitted_name`` is ['a', 'b', 'c'] but only ``a.b`` is
    importable, this function:

        1. tries to import ``a.b.c`` and fails
        2. tries to import ``a.b`` and succeeds
        3. return ``a.b`` and the exception that occurred at step 1.
    """
    module = None
    import_error = None
    while splitted_name:
        try:
            module = __import__(".".join(splitted_name))
            break
        except BaseException:
            import_error = sys.exc_info()
            del splitted_name[-1]
            if not splitted_name:
                raise
    return (module, import_error)


def name_from_args(name, index, args):
    """Create test name from test args"""
    summary = ", ".join(repr(arg) for arg in args)
    return f"{name}:{index + 1}\n{summary[:79]}"


def test_name(test, qualname=True):
    # XXX does not work for test funcs; test.id() lacks module
    if hasattr(test, "_funcName"):
        tid = test._funcName
    elif hasattr(test, "_testFunc"):
        tid = f"{test._testFunc.__module__}.{test._testFunc.__name__}"
    else:
        if not qualname:
            test_module = test.__class__.__module__
            test_class = test.__class__.__name__
            test_method = test._testMethodName
            tid = f"{test_module}.{test_class}.{test_method}"
        else:
            tid = test.id()
    if "\n" in tid:
        tid = tid.split("\n")[0]
    # subtest support
    if " " in tid:
        tid = tid.split(" ")[0]
    return tid


def ispackage(path):
    """Is this path a package directory?"""
    if os.path.isdir(path):
        # at least the end of the path must be a legal python identifier
        # and __init__.py[co] must exist
        end = os.path.basename(path)
        if IDENT_RE.match(end):
            for init in ("__init__.py", "__init__.pyc", "__init__.pyo"):
                if os.path.isfile(os.path.join(path, init)):
                    return True
            if sys.platform.startswith("java") and os.path.isfile(
                os.path.join(path, "__init__$py.class")
            ):
                return True
    return False


def ensure_importable(dirname):
    """Ensure a directory is on ``sys.path``."""
    if dirname not in sys.path:
        sys.path.insert(0, dirname)


def has_module_fixtures(test):
    """Does this test live in a module with module fixtures?"""
    # test may be class or instance
    test_class = test if isinstance(test, type) else test.__class__
    modname = test_class.__module__
    try:
        mod = sys.modules[modname]
    except KeyError:
        return
    return hasattr(mod, "setUpModule") or hasattr(mod, "tearDownModule")


def has_class_fixtures(test):
    # test may be class or instance
    test_class = test if isinstance(test, type) else test.__class__

    # hasattr would be the obvious thing to use here. Unfortunately, all tests
    # inherit from unittest2.case.TestCase, and that *always* has setUpClass and
    # tearDownClass methods. Thus, exclude the unitest and unittest2 base
    # classes from the lookup.
    def is_not_base_class(c):
        return (
            "unittest.case" not in c.__module__ and "unittest2.case" not in c.__module__
        )

    has_class_setups = any(
        "setUpClass" in c.__dict__ for c in test_class.__mro__ if is_not_base_class(c)
    )
    has_class_teardowns = any(
        "tearDownClass" in c.__dict__
        for c in test_class.__mro__
        if is_not_base_class(c)
    )
    return has_class_setups or has_class_teardowns


def safe_decode(string):
    """Safely decode a byte string into unicode"""
    if string is None:
        return string
    try:
        return string.decode()
    except AttributeError:
        return string
    except UnicodeDecodeError:
        pass
    try:
        return string.decode("utf-8")
    except UnicodeDecodeError:
        return "<unable to decode>"


def exc_info_to_string(err, test):
    """Format exception info for output"""
    formatTraceback = getattr(test, "formatTraceback", None)
    if formatTraceback is not None:
        return test.formatTraceback(err)
    else:
        return format_traceback(test, err)


def format_traceback(test, err):
    """Converts a :func:`sys.exc_info` -style tuple of values into a string."""
    exctype, value, tb = err
    if not hasattr(tb, "tb_next"):
        msgLines = tb
    else:
        # Skip test runner traceback levels
        while tb and _is_relevant_tb_level(tb):
            tb = tb.tb_next
        failure = getattr(test, "failureException", AssertionError)
        if exctype is failure:
            # Skip assert*() traceback levels
            length = _count_relevant_tb_levels(tb)
            msgLines = traceback.format_exception(exctype, value, tb, length)
        else:
            msgLines = traceback.format_exception(exctype, value, tb)

    return "".join(msgLines)


def transplant_class(cls, module):
    """Make ``cls`` appear to reside in ``module``.

    :param cls: A class
    :param module: A module name
    :returns: A subclass of ``cls`` that appears to have been defined in ``module``.

    The returned class's ``__name__`` will be equal to
    ``cls.__name__``, and its ``__module__`` equal to ``module``.

    """

    class C(cls):
        pass

    C.__module__ = module
    C.__name__ = cls.__name__
    return C


def parse_log_level(lvl):
    """Return numeric log level given a string"""
    try:
        return int(lvl)
    except ValueError:
        pass
    return getattr(logging, lvl.upper(), logging.WARN)


def _is_relevant_tb_level(tb):
    return "__unittest" in tb.tb_frame.f_globals


def _count_relevant_tb_levels(tb):
    length = 0
    while tb and not _is_relevant_tb_level(tb):
        length += 1
        tb = tb.tb_next
    return length


class _WritelnDecorator:

    """Used to decorate file-like objects with a handy :func:`writeln` method"""

    def __init__(self, stream):
        self.stream = stream

    def __getattr__(self, attr):
        if attr in ("stream", "__getstate__"):
            raise AttributeError(attr)
        return getattr(self.stream, attr)

    def write(self, arg):
        self.stream.write(arg)

    def writeln(self, arg=None):
        if arg:
            self.write(arg)
        self.write("\n")  # text-mode streams translate to \r\n if needed


def ancestry(layer):
    layers = [[layer]]
    bases = [base for base in bases_and_mixins(layer) if base is not object]
    while bases:
        layers.append(bases)
        newbases = []
        for b in bases:
            for bb in bases_and_mixins(b):
                if bb is not object:
                    newbases.append(bb)
        bases = newbases
    layers.reverse()
    return layers


def bases_and_mixins(layer):
    return layer.__bases__ + getattr(layer, "mixins", ())


def num_expected_args(func):
    """Return the number of arguments that :func: expects"""
    return len(inspect.getfullargspec(func)[0])


def call_with_args_if_expected(func, *args):
    """Take :func: and call it with supplied :args:, in case that signature expects any.
    Otherwise call the function without any arguments.
    """
    if num_expected_args(func) > 0:
        func(*args)
    else:
        func()


def iter_attrs(obj):
    # note: vars() can be surprising __slots__ is defined
    # using dir(x) is the most general way of doing this
    for attrname in dir(obj):
        yield (attrname, getattr(obj, attrname))



import time

from nose2 import events

ERROR = "error"
FAIL = "failed"
SKIP = "skipped"
PASS = "passed"
SUBTEST = "subtest"
__unittest = True


class PluggableTestResult:

    """Test result that defers to plugins.

    All test outcome recording and reporting is deferred to plugins,
    which are expected to implement :func:`startTest`, :func:`stopTest`,
    :func:`testOutcome`, and :func:`wasSuccessful`.

    :param session: Test run session.

    .. attribute :: shouldStop

       When ``True``, test run should stop before running another test.

    """

    def __init__(self, session):
        self.session = session
        self.shouldStop = False
        # XXX TestCase.subTest expects a result.failfast attribute
        self.failfast = False

    def startTest(self, test):
        """Start a test case.

        Fires :func:`startTest` hook.

        """
        event = events.StartTestEvent(test, self, time.time())
        self.session.hooks.startTest(event)

    def stopTest(self, test):
        """Stop a test case.

        Fires :func:`stopTest` hook.

        """
        event = events.StopTestEvent(test, self, time.time())
        self.session.hooks.stopTest(event)

    def addDuration(self, test, elapsed):  # For Python >= 3.12
        pass

    def addError(self, test, err):
        """Test case resulted in error.

        Fires :func:`setTestOutcome` and :func:`testOutcome` hooks.

        """
        event = events.TestOutcomeEvent(test, self, ERROR, err)
        self.session.hooks.setTestOutcome(event)
        self.session.hooks.testOutcome(event)

    def addFailure(self, test, err):
        """Test case resulted in failure.

        Fires :func:`setTestOutcome` and :func:`testOutcome` hooks.

        """
        event = events.TestOutcomeEvent(test, self, FAIL, err)
        self.session.hooks.setTestOutcome(event)
        self.session.hooks.testOutcome(event)

    def addSubTest(self, test, subtest, err):
        """Called at the end of a subtest.

        Fires :func:`setTestOutcome` and :func:`testOutcome` hooks.

        """
        event = events.TestOutcomeEvent(subtest, self, SUBTEST, err)
        self.session.hooks.setTestOutcome(event)
        self.session.hooks.testOutcome(event)

    def addSuccess(self, test):
        """Test case resulted in success.

        Fires :func:`setTestOutcome` and :func:`testOutcome` hooks.

        """
        event = events.TestOutcomeEvent(test, self, PASS, expected=True)
        self.session.hooks.setTestOutcome(event)
        self.session.hooks.testOutcome(event)

    def addSkip(self, test, reason):
        """Test case was skipped.

        Fires :func:`setTestOutcome` and :func:`testOutcome` hooks.

        """
        event = events.TestOutcomeEvent(test, self, SKIP, reason=reason)
        self.session.hooks.setTestOutcome(event)
        self.session.hooks.testOutcome(event)

    def addExpectedFailure(self, test, err):
        """Test case resulted in expected failure.

        Fires :func:`setTestOutcome` and :func:`testOutcome` hooks.

        """
        event = events.TestOutcomeEvent(test, self, FAIL, err, expected=True)
        self.session.hooks.setTestOutcome(event)
        self.session.hooks.testOutcome(event)

    def addUnexpectedSuccess(self, test):
        """Test case resulted in unexpected success.

        Fires :func:`setTestOutcome` and :func:`testOutcome` hooks.

        """
        event = events.TestOutcomeEvent(test, self, PASS)
        self.session.hooks.setTestOutcome(event)
        self.session.hooks.testOutcome(event)

    def wasSuccessful(self):
        """Was test run successful?

        Fires :func:`wasSuccessful` hook, and returns ``event.success``.

        """
        # assume failure; plugins must explicitly declare success
        try:
            return self._success
        except AttributeError:
            event = events.ResultSuccessEvent(self, None)
            self.session.hooks.wasSuccessful(event)
            self._success = event.success
            return self._success

    def stop(self):
        """Stop test run.

        Fires :func:`resultStop` hook, and sets ``self.shouldStop`` to
        ``event.shouldStop``.

        """
        event = events.ResultStopEvent(self, True)
        self.session.hooks.resultStop(event)
        self.shouldStop = event.shouldStop

    def __repr__(self):
        return "<%s>" % self.__class__.__name__



"""
Make assert statements print pretty output, including source.

This makes ``assert x == y`` more usable, as an alternative to
``self.assertEqual(x, y)``

This plugin implements :func:`outcomeDetail` and checks for event.exc_info
If it finds that an AssertionError happened, it will inspect the traceback and
add additional detail to the error report.

"""


import collections
import inspect
import io
import re
import textwrap
import tokenize

from nose2 import events

__unittest = True


class PrettyAssert(events.Plugin):
    """Add pretty output for "assert" statements"""

    configSection = "pretty-assert"
    commandLineSwitch = (
        None,
        "pretty-assert",
        'Add pretty output for "assert" statements',
    )

    def outcomeDetail(self, event):
        # skip if no exception or expected error
        if (not event.outcomeEvent.exc_info) or event.outcomeEvent.expected:
            return

        # unpack, but skip if it's not an AssertionError
        excty, exc, trace = event.outcomeEvent.exc_info
        if excty is not AssertionError:
            return

        self.addAssertDetail(event.extraDetail, exc, trace)

    @staticmethod
    def addAssertDetail(extraDetail, exc, trace):
        """
        Add details to output regarding AssertionError and its context

        extraDetail: a list of lines which will be joined with newlines and
        added to the output for this test failure -- defined as part of the
        event format

        exc: the AssertionError exception which was thrown

        trace: a traceback object for the exception
        """
        assert_statement, token_descriptions = _collect_assert_data(trace)

        # no message was given
        if len(exc.args) == 0:
            message = None
        else:
            message = exc.args[0]

        # if there is no assertion statement found, do not add detail to output
        #
        # in cases like unittest assert*() methods, an assertion error is
        # raised, but it doesn't originate with an `assert` statement and has
        # an autogenerated message
        if not assert_statement:
            return

        #
        # actually add exception info to detail
        #

        # add the assert statement to output with '>>>' prefix
        extraDetail.append(re.sub("^", ">>> ", assert_statement, flags=re.MULTILINE))

        if message:
            extraDetail.append("\nmessage:")
            extraDetail.append(f"    {message}")

        if token_descriptions:
            extraDetail.append("\nvalues:")
            for k, v in token_descriptions.items():
                extraDetail.append(f"    {k} = {v}")


def _collect_assert_data(trace):
    """
    Given a traceback, extract the assertion statement and get the set of bound
    variable names (i.e. tokens)
    """
    # inspect the trace, collecting various data and determining whether or not
    # it can be tokenized at all
    source_lines, frame_locals, frame_globals, can_tokenize = _get_inspection_info(
        trace
    )

    # if things will tokenize cleanly, actually do it
    if can_tokenize:
        assert_startline, token_descriptions = _tokenize_assert(
            source_lines, frame_locals, frame_globals
        )
    # otherwise, indicate that we can't render detail by use of Nones
    else:
        assert_startline = None
        token_descriptions = None

    # if we found an "assert" (we might not, if someone raises
    # AssertionError themselves), grab the whole assertion statement
    #
    # as a fallback, stick with whatever we think the statement was
    # - this is easily deceived by multiline expressions
    if assert_startline is not None:
        statement = textwrap.dedent(
            "".join(source_lines[assert_startline:]).rstrip("\n")
        )
    else:
        statement = None

    return statement, token_descriptions


def _get_inspection_info(trace):
    """
    Pick apart a traceback for the info we actually want to inspect from it
    - lines of source (truncated)
    - locals and globals from execution frame
    - statement which failed (which can be garbage -- don't trust it)
    - can_tokenize: a bool indicating that the lines of source can be parsed
    """
    (frame, fname, lineno, funcname, context, ctx_index) = inspect.getinnerframes(
        trace
    )[-1]
    original_source_lines, firstlineno = inspect.getsourcelines(frame)

    # truncate to the code in this frame to remove anything after current
    # assert statement
    last_index = lineno - firstlineno + 1
    source_lines = original_source_lines[:last_index]

    # in case the current line is actually an incomplete expression, as in
    #   assert x == (y
    #                ).z
    #
    # in which case the the current line is "assert x == (y", which is not a
    # complete expression
    # try to append lines to complete the expression, retrying parsing each and
    # every time until it succeeds
    for line in original_source_lines[last_index:]:
        if _can_tokenize(source_lines):
            break
        else:
            source_lines.append(line)

    return (source_lines, frame.f_locals, frame.f_globals, _can_tokenize(source_lines))


def _can_tokenize(source_lines):
    """
    Check if a list of lines of source can successfully be tokenized
    """
    # tokenize.generate_tokens requires a file-like object, so we need to
    # convert source_lines to a StringIO to give it that interface
    filelike = io.StringIO(textwrap.dedent("".join(source_lines)))

    try:
        for _tokty, _tok, _start, _end, _tok_lineno in tokenize.generate_tokens(
            filelike.readline
        ):
            pass
    except tokenize.TokenError:
        return False

    return True


def _tokenize_assert(source_lines, frame_locals, frame_globals):
    """
    Given a set of lines of source ending in a failing assert, plus the frame
    locals and globals, tokenize source.

    Only look at tokens in the final assert statement
    Resolve all names to repr() of values

    Return
        The line on which the assert starts (relative to start of
        source_lines)

        A collection of token descriptions as a name=val ordered dict
    """
    # tokenize.generate_tokens requires a file-like object, so we need to
    # convert source_lines to a StringIO to give it that interface
    filelike_context = io.StringIO(textwrap.dedent("".join(source_lines)))

    # track the first line of the assert statement
    # when the assert is on oneline, we'll have it easily, but a multiline
    # statement like
    #   assert (x ==
    #           1)
    # will leave us holding the last line of the statement,
    # e.g. "       1)", which is not useful
    # so every time a new assert is found, we get a value back indicate
    # that it's the start line
    #
    #   assert True
    #   assert False
    # works fine, because we'll just hold the last value
    #
    #   assert True
    #   assert False
    #   assert True
    # also works because we truncated source_lines to remove the final
    # assert, which we didn't reach during execution
    assert_startline = None

    token_processor = TokenProcessor(frame_locals, frame_globals)

    # tokenize and process each token
    for tokty, tok, start, end, tok_lineno in tokenize.generate_tokens(
        filelike_context.readline
    ):
        ret = token_processor.handle_token(tokty, tok, start, end, tok_lineno)
        if ret:
            assert_startline = ret

    # adjust assert_startline by 1 to become a valid index into the
    # source_lines -- "line 1" means "index 0"
    if assert_startline:
        assert_startline -= 1

    token_descriptions = collections.OrderedDict()
    for name, obj in token_processor.get_token_collection().items():
        # okay, get repr() for a good string representation
        strvalue = repr(obj)
        # add in the form we want to print
        token_descriptions[name] = strvalue

    return assert_startline, token_descriptions


class TokenProcessor:
    def __init__(self, frame_locals, frame_globals):
        # local and global variables from the frame which we're inspecting
        self.frame_locals, self.frame_globals = frame_locals, frame_globals

        # None or a tuple of (object, name) where
        # - "object" is the object whose attributes we are currently resolving
        # - "name" is its name, as we would like to display it
        #
        # start each time we see a sequence of NAME OP NAME OP NAME (etc.)
        # end each time we see a token which is neither NAME nor OP
        self.doing_resolution = None

        # an index of known token names (including the long "x.y.z" names we
        # get from attribute resolution) to their values, in the order in which
        # they were encountered
        # track which tokens we've seen to avoid duplicates if a name appears
        # twice, as in `assert x != x`
        self.seen_tokens = collections.OrderedDict()

        # the previous token seen as a tuple of (tok_type, token_name)
        # (or None when we start)
        self.last_tok = None

    def get_token_collection(self):
        return self.seen_tokens

    def handle_token(self, toktype, tok, start, end, line):
        """
        A tokenization processor for tokenize.generate_tokens
        Skips certain token types, class names, etc

        When an identifiable/usable token is found, add it to the token
        collection (self.seen_tokens)

        When an "assert" statement is found, reset the token collection
        and return the start line (relative to the text being tokenized)
        """
        prior_tok = self.last_tok
        self.last_tok = (toktype, tok)

        # CASE 0: skip non "NAME" or "OP" tokens and clear current resolution
        #
        # NAME is most identifiers and keywords
        # OP is operators, including .
        #
        # special note: don't clear resolution for whitespace (e.g. newline)
        if toktype not in (tokenize.NAME, tokenize.OP):
            # only newline for now, maybe we'll find others
            if toktype not in (tokenize.NEWLINE,):
                self.doing_resolution = None
            return

        # CASE 1: Operator token
        #
        # skip tokens and either leave resolution in progress or reset,
        # depending
        #
        # continue resolution for
        #   "."
        #     because that's what attribute resolution *is*
        #   ")"
        #     this is handy, as it means that "(x).y" works
        #
        # reset resolution for everything else, e.g. "[", "]", ":"
        # special note: reset resolution for "("
        #
        # failing to filter out "(" can result in badness in cases like this:
        #     >>> def foo():
        #     >>>     return [1]
        #     >>> foo.pop = 2
        #     >>> ...
        #     >>> def test_foo():
        #     >>>    assert foo().pop() == 2
        #
        # if we stop resolution when we see an LPAREN, we resolve `foo`
        # successfully, fail on `pop` and everything is OK, but if we try to
        # traverse the LPAREN, we get `foo.pop = 2` in our values, which is
        # wrong
        if toktype == tokenize.OP:
            if tok not in (".", ")"):
                self.doing_resolution = None
            return

        # CASE 2: "assert" statement
        # assert statement was reached, reset
        # return the start line (start = (startrow, startcol))
        if tok == "assert":
            self.seen_tokens.clear()
            self.doing_resolution = None
            return start[0]

        # handle tokens

        # CASE 3: a name is being resolved,
        #         there is a previous token,
        #         and it's a "." operator
        if (
            self.doing_resolution
            and prior_tok
            and (prior_tok[0] == tokenize.OP and prior_tok[1] == ".")
        ):
            # unpack and look for the attribute
            obj, name = self.doing_resolution
            if hasattr(obj, tok):
                obj = getattr(obj, tok)
                name = name + "." + tok
                self.doing_resolution = (obj, name)
                self.seen_tokens[name] = obj
            # if we couldn't find a relevant attribute, reset on resolution so
            # that we can try afresh
            else:
                self.doing_resolution = None

        # CASE 4: a name is being resolved and there is no preceding "." or
        #         resolution was explicitly stopped
        else:
            # skip tokens we've seen, but grab them as the current things under
            # resolution
            if tok in self.seen_tokens:
                self.doing_resolution = (self.seen_tokens[tok], tok)
                return
            # we've never seen this token before
            else:
                # try to resolve to a value
                try:
                    value = self.frame_locals[tok]
                except KeyError:
                    try:
                        value = self.frame_globals[tok]
                    except KeyError:
                        # unresolveable name -- short circuit
                        # shows up in some cases like `f().x` in which `x`
                        # might not be a name bound to a value
                        return

                # add it (so we don't try it again unless we hit a new assert
                # and reset)
                self.seen_tokens[tok] = value

                self.doing_resolution = (value, tok)



import re

from nose2.tests._common import FunctionalTestCase


class CollectOnlyFunctionalTest(FunctionalTestCase):
    def test_collect_tests_in_package(self):
        self.assertTestRunOutputMatches(
            self.runIn(
                "scenario/tests_in_package",
                "-v",
                "--collect-only",
                "--plugin=nose2.plugins.collect",
            ),
            stderr=EXPECT_LAYOUT1,
        )


# expectations
EXPECT_LAYOUT1 = re.compile(
    r"""Ran 25 tests in \d.\d+s

OK"""
)



"""Test testid plugin."""
import io
import os.path
import pickle

from nose2 import session
from nose2.events import ReportTestEvent
from nose2.plugins import testid
from nose2.tests._common import (
    FakeLoadFromNameEvent,
    FakeLoadFromNamesEvent,
    FakeStartTestEvent,
    TestCase,
)


class UnitTestTestId(TestCase):

    """Test class TestId.

    Tests are carried out in a temporary directory, since TestId stores state
    to file. The temporary directory is removed after testing.
    """

    tags = ["unit"]
    _RUN_IN_TEMP = True

    def setUp(self):
        super().setUp()
        self.stream = io.StringIO()
        self.session = session.Session()
        self.plugin = testid.TestId(session=self.session)

    def test___init__(self):
        """Test the __init__ method."""
        plug = self.plugin
        # Test attributes
        for name, exp_val in [
            ("configSection", "testid"),
            ("commandLineSwitch", ("I", "with-id", "Add test ids to output")),
            ("idfile", os.path.abspath(".noseids")),
            ("ids", {}),
            ("tests", {}),
            ("id", 0),
        ]:
            try:
                val = getattr(plug, name)
            except AttributeError:
                self.fail(f"TestId instance doesn't have attribute {name}")
            self.assertEqual(
                val,
                exp_val,
                "Attribute {} should have value "
                "'{}', but has value {}".format(name, exp_val, val),
            )

    def test_start_test(self):
        """Test reportStartTest method."""
        self.session.verbosity = 2
        event = ReportTestEvent(FakeStartTestEvent(self), self.stream)
        plug = self.plugin
        plug.reportStartTest(event)

        self.assertEqual(plug.id, 1)
        test_id = self.id()
        self.assertEqual(plug.ids, {1: test_id})
        self.assertEqual(plug.tests, {test_id: 1})
        self.assertEqual(self.stream.getvalue(), "#1 ")

    def test_start_test_twice(self):
        """Test calling reportStartTest twice."""
        self.session.verbosity = 2
        event = ReportTestEvent(FakeStartTestEvent(self), self.stream)
        plug = self.plugin
        plug.reportStartTest(event)
        plug.reportStartTest(event)

        self.assertEqual(plug.id, 1)
        test_id = self.id()
        self.assertEqual(plug.ids, {1: test_id})
        self.assertEqual(plug.tests, {test_id: 1})
        self.assertEqual(self.stream.getvalue(), "#1 #1 ")

    def test_stop_test_run(self):
        """Test stopTestRun method."""
        plug = self.plugin
        plug.reportStartTest(ReportTestEvent(FakeStartTestEvent(self), self.stream))
        plug.stopTestRun(None)

        fh = open(plug.idfile, "rb")
        try:
            data = pickle.load(fh)
        finally:
            fh.close()
        self.assertEqual(data, {"ids": plug.ids, "tests": plug.tests})

    def test_load_tests_from_name(self):
        """Test loadTestsFromName method."""
        plug = self.plugin
        # By first starting/stopping a test, an ID is assigned by the plugin
        plug.reportStartTest(ReportTestEvent(FakeStartTestEvent(self), self.stream))
        plug.stopTestRun(None)
        event = FakeLoadFromNameEvent("1")
        plug.loadTestsFromName(event)

        # The numeric ID should be translated to this test's ID
        self.assertEqual(event.name, self.id())

    def test_load_tests_from_name_no_ids(self):
        """Test calling loadTestsFromName when no IDs have been saved."""
        plug = self.plugin
        event = FakeLoadFromNameEvent("1")
        plug.loadTestsFromName(event)

        # The event's name should be unchanged, since no IDs should be mapped
        self.assertEqual(event.name, "1")

    def test_load_tests_from_names(self):
        """Test loadTestsFromNames method."""
        plug = self.plugin
        # By first starting/stopping a test, an ID is assigned by the plugin
        plug.reportStartTest(ReportTestEvent(FakeStartTestEvent(self), self.stream))
        plug.stopTestRun(None)
        event = FakeLoadFromNamesEvent(["1", "2"])
        plug.loadTestsFromNames(event)

        name1, name2 = event.names
        # The first numeric ID should be translated to this test's ID
        self.assertEqual(name1, self.id())
        # The second one should not have a match
        self.assertEqual(name2, "2")



import time

from nose2.tools import such


def slow_blocking_init():
    print("YEAH2")
    time.sleep(1)
    print("a second elapsed")
    time.sleep(1)
    print("a second elapsed")
    return True


class Layer1:
    description = "Layer1 description"

    @classmethod
    def setUp(cls):
        print("YEAH")
        it.obj = False


class Layer2:
    description = "Layer2 description"

    @classmethod
    def setUp(cls):
        it.obj = slow_blocking_init()


with such.A("system with a fast initial setup layer") as it:
    it.uses(Layer1)

    @it.should("not have obj initialized")
    def test():
        assert not it.obj

    with it.having("a second slow setup layer"):
        it.uses(Layer2)

        @it.should("have obj initialized")
        def test2():
            assert it.obj


it.createTests(globals())



import logging

from nose2.tools import such

from .common import NormalTest, NormalTestTwo, UniqueResource  # noqa: F401

log = logging.getLogger(__name__)


class Layer1:
    description = "Layer1"

    @classmethod
    def setUp(cls):
        log.info("Called setup in layer 1")
        it.unique_resource = UniqueResource()
        it.unique_resource.lock()

    @classmethod
    def tearDown(cls):
        log.info("Called teardown in layer 2")
        it.unique_resource.unlock()


class Layer2:
    description = "Layer2"

    @classmethod
    def setUp(cls):
        log.info("Called setup in layer 2")

    @classmethod
    def tearDown(cls):
        log.info("Called teardown in layer 2")


with such.A("system with setup") as it:
    it.uses(Layer1)

    @it.should("do something")
    def test(case):
        it.assertTrue(it.unique_resource.used)

    with it.having("another setup"):
        it.uses(Layer2)

        @it.should("do something else")  # noqa: F811
        def test(case):  # noqa: F811
            it.assertTrue(it.unique_resource.used)


it.createTests(globals())



import unittest


class Layer:
    description = "fixture with a value"

    @classmethod
    def setUp(cls):
        cls.value = 1


class Test(unittest.TestCase):
    layer = Layer

    def test_ok(self):
        self.assertEqual(self.layer.value, 1)

    def test_fail(self):
        self.assertEqual(self.layer.value, 2)

    def test_err(self):
        self.assertEqual(self.layer.mulch, "pine")



print("import: 'nose2'")
import nose2

print("import: 'nose2.plugins'")
import nose2.plugins

print("import: 'nose2.plugins.loader'")
import nose2.plugins.loader

print("import: 'nose2.tests'")
import nose2.tests

print("import: 'nose2.tests.functional'")
import nose2.tests.functional

print("import: 'nose2.tests.unit'")
import nose2.tests.unit

print("import: 'nose2.tools'")
import nose2.tools




import os
import sys

from nose2.tests._common import FunctionalTestCase


class TestLayerHooks(FunctionalTestCase):
    @classmethod
    def setUp(cls):
        filepath = os.path.dirname(os.path.realpath(__file__))
        cls.libpath = os.path.join(filepath, "support", "lib")
        sys.path.append(cls.libpath)

    @classmethod
    def tearDown(cls):
        if cls.libpath in sys.path:
            sys.path.remove(cls.libpath)

    def test_simple_such(self):
        proc = self.runIn(
            "scenario/layers_hooks",
            "--plugin=nose2.plugins.layers",
            "--plugin=layer_hooks_plugin",
            "test_simple_such",
        )
        if sys.version_info >= (3, 11):
            expected = r"""^StartLayerSetup: <class 'test_simple_such.A system:layer'>
StopLayerSetup: <class 'test_simple_such.A system:layer'>
StartLayerSetupTest: <class 'test_simple_such.A system:layer'>:test 0000: should do something \(test_simple_such.A system.test 0000: should do something\)
StopLayerSetupTest: <class 'test_simple_such.A system:layer'>:test 0000: should do something \(test_simple_such.A system.test 0000: should do something\)
StartLayerTeardownTest: <class 'test_simple_such.A system:layer'>:test 0000: should do something \(test_simple_such.A system.test 0000: should do something\)
StopLayerTeardownTest: <class 'test_simple_such.A system:layer'>:test 0000: should do something \(test_simple_such.A system.test 0000: should do something\)
StartLayerTeardown: <class 'test_simple_such.A system:layer'>
StopLayerTeardown: <class 'test_simple_such.A system:layer'>\n*$"""  # noqa: E501
        else:
            expected = r"""^StartLayerSetup: <class 'test_simple_such.A system:layer'>
StopLayerSetup: <class 'test_simple_such.A system:layer'>
StartLayerSetupTest: <class 'test_simple_such.A system:layer'>:test 0000: should do something \(test_simple_such.A system\)
StopLayerSetupTest: <class 'test_simple_such.A system:layer'>:test 0000: should do something \(test_simple_such.A system\)
StartLayerTeardownTest: <class 'test_simple_such.A system:layer'>:test 0000: should do something \(test_simple_such.A system\)
StopLayerTeardownTest: <class 'test_simple_such.A system:layer'>:test 0000: should do something \(test_simple_such.A system\)
StartLayerTeardown: <class 'test_simple_such.A system:layer'>
StopLayerTeardown: <class 'test_simple_such.A system:layer'>\n*$"""  # noqa: E501
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stdout=expected)
        self.assertEqual(proc.poll(), 0, proc.stderr.getvalue())

    def test_complex_such(self):
        proc = self.runIn(
            "such/",
            "--plugin=nose2.plugins.layers",
            "--plugin=layer_hooks_plugin",
            "test_such",
        )
        if sys.version_info >= (3, 11):
            expected = r"""StartLayerSetup: <class 'test_such.A system with complex setup:layer'>
StopLayerSetup: <class 'test_such.A system with complex setup:layer'>
StartLayerSetupTest: <class 'test_such.A system with complex setup:layer'>:test 0000: should do something \(test_such.A system with complex setup.test 0000: should do something\)
StopLayerSetupTest: <class 'test_such.A system with complex setup:layer'>:test 0000: should do something \(test_such.A system with complex setup.test 0000: should do something\)
StartLayerTeardownTest: <class 'test_such.A system with complex setup:layer'>:test 0000: should do something \(test_such.A system with complex setup.test 0000: should do something\)
StopLayerTeardownTest: <class 'test_such.A system with complex setup:layer'>:test 0000: should do something \(test_such.A system with complex setup.test 0000: should do something\)
StartLayerSetup: <class 'test_such.having an expensive fixture:layer'>
StopLayerSetup: <class 'test_such.having an expensive fixture:layer'>
StartLayerSetupTest: <class 'test_such.having an expensive fixture:layer'>:test 0000: should do more things \(test_such.having an expensive fixture.test 0000: should do more things\)
StopLayerSetupTest: <class 'test_such.having an expensive fixture:layer'>:test 0000: should do more things \(test_such.having an expensive fixture.test 0000: should do more things\)
StartLayerTeardownTest: <class 'test_such.having an expensive fixture:layer'>:test 0000: should do more things \(test_such.having an expensive fixture.test 0000: should do more things\)
StopLayerTeardownTest: <class 'test_such.having an expensive fixture:layer'>:test 0000: should do more things \(test_such.having an expensive fixture.test 0000: should do more things\)
StartLayerSetup: <class 'test_such.having another precondition:layer'>
StopLayerSetup: <class 'test_such.having another precondition:layer'>
StartLayerSetupTest: <class 'test_such.having another precondition:layer'>:test 0000: should do that not this \(test_such.having another precondition.test 0000: should do that not this\)
StopLayerSetupTest: <class 'test_such.having another precondition:layer'>:test 0000: should do that not this \(test_such.having another precondition.test 0000: should do that not this\)
StartLayerTeardownTest: <class 'test_such.having another precondition:layer'>:test 0000: should do that not this \(test_such.having another precondition.test 0000: should do that not this\)
StopLayerTeardownTest: <class 'test_such.having another precondition:layer'>:test 0000: should do that not this \(test_such.having another precondition.test 0000: should do that not this\)
StartLayerSetupTest: <class 'test_such.having another precondition:layer'>:test 0001: should do this not that \(test_such.having another precondition.test 0001: should do this not that\)
StopLayerSetupTest: <class 'test_such.having another precondition:layer'>:test 0001: should do this not that \(test_such.having another precondition.test 0001: should do this not that\)
StartLayerTeardownTest: <class 'test_such.having another precondition:layer'>:test 0001: should do this not that \(test_such.having another precondition.test 0001: should do this not that\)
StopLayerTeardownTest: <class 'test_such.having another precondition:layer'>:test 0001: should do this not that \(test_such.having another precondition.test 0001: should do this not that\)
StartLayerTeardown: <class 'test_such.having another precondition:layer'>
StopLayerTeardown: <class 'test_such.having another precondition:layer'>
StartLayerSetup: <class 'test_such.SomeLayer'>
StopLayerSetup: <class 'test_such.SomeLayer'>
StartLayerSetup: <class 'test_such.having a different precondition:layer'>
StopLayerSetup: <class 'test_such.having a different precondition:layer'>
StartLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0000: should do something else \(test_such.having a different precondition.test 0000: should do something else\)
StopLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0000: should do something else \(test_such.having a different precondition.test 0000: should do something else\)
StartLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0000: should do something else \(test_such.having a different precondition.test 0000: should do something else\)
StopLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0000: should do something else \(test_such.having a different precondition.test 0000: should do something else\)
StartLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0001: should have another test \(test_such.having a different precondition.test 0001: should have another test\)
StopLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0001: should have another test \(test_such.having a different precondition.test 0001: should have another test\)
StartLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0001: should have another test \(test_such.having a different precondition.test 0001: should have another test\)
StopLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0001: should have another test \(test_such.having a different precondition.test 0001: should have another test\)
StartLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0002: should have access to an external fixture \(test_such.having a different precondition.test 0002: should have access to an external fixture\)
StopLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0002: should have access to an external fixture \(test_such.having a different precondition.test 0002: should have access to an external fixture\)
StartLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0002: should have access to an external fixture \(test_such.having a different precondition.test 0002: should have access to an external fixture\)
StopLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0002: should have access to an external fixture \(test_such.having a different precondition.test 0002: should have access to an external fixture\)
StartLayerSetup: <class 'test_such.having a case inside the external fixture:layer'>
StopLayerSetup: <class 'test_such.having a case inside the external fixture:layer'>
StartLayerSetupTest: <class 'test_such.having a case inside the external fixture:layer'>:test 0000: should still have access to that fixture \(test_such.having a case inside the external fixture.test 0000: should still have access to that fixture\)
StopLayerSetupTest: <class 'test_such.having a case inside the external fixture:layer'>:test 0000: should still have access to that fixture \(test_such.having a case inside the external fixture.test 0000: should still have access to that fixture\)
StartLayerTeardownTest: <class 'test_such.having a case inside the external fixture:layer'>:test 0000: should still have access to that fixture \(test_such.having a case inside the external fixture.test 0000: should still have access to that fixture\)
StopLayerTeardownTest: <class 'test_such.having a case inside the external fixture:layer'>:test 0000: should still have access to that fixture \(test_such.having a case inside the external fixture.test 0000: should still have access to that fixture\)
StartLayerTeardown: <class 'test_such.having a case inside the external fixture:layer'>
StopLayerTeardown: <class 'test_such.having a case inside the external fixture:layer'>
StartLayerTeardown: <class 'test_such.having a different precondition:layer'>
StopLayerTeardown: <class 'test_such.having a different precondition:layer'>
StartLayerTeardown: <class 'test_such.SomeLayer'>
StopLayerTeardown: <class 'test_such.SomeLayer'>
StartLayerTeardown: <class 'test_such.having an expensive fixture:layer'>
StopLayerTeardown: <class 'test_such.having an expensive fixture:layer'>
StartLayerTeardown: <class 'test_such.A system with complex setup:layer'>
StopLayerTeardown: <class 'test_such.A system with complex setup:layer'>\n*$"""  # noqa: E501
        else:
            expected = r"""StartLayerSetup: <class 'test_such.A system with complex setup:layer'>
StopLayerSetup: <class 'test_such.A system with complex setup:layer'>
StartLayerSetupTest: <class 'test_such.A system with complex setup:layer'>:test 0000: should do something \(test_such.A system with complex setup\)
StopLayerSetupTest: <class 'test_such.A system with complex setup:layer'>:test 0000: should do something \(test_such.A system with complex setup\)
StartLayerTeardownTest: <class 'test_such.A system with complex setup:layer'>:test 0000: should do something \(test_such.A system with complex setup\)
StopLayerTeardownTest: <class 'test_such.A system with complex setup:layer'>:test 0000: should do something \(test_such.A system with complex setup\)
StartLayerSetup: <class 'test_such.having an expensive fixture:layer'>
StopLayerSetup: <class 'test_such.having an expensive fixture:layer'>
StartLayerSetupTest: <class 'test_such.having an expensive fixture:layer'>:test 0000: should do more things \(test_such.having an expensive fixture\)
StopLayerSetupTest: <class 'test_such.having an expensive fixture:layer'>:test 0000: should do more things \(test_such.having an expensive fixture\)
StartLayerTeardownTest: <class 'test_such.having an expensive fixture:layer'>:test 0000: should do more things \(test_such.having an expensive fixture\)
StopLayerTeardownTest: <class 'test_such.having an expensive fixture:layer'>:test 0000: should do more things \(test_such.having an expensive fixture\)
StartLayerSetup: <class 'test_such.having another precondition:layer'>
StopLayerSetup: <class 'test_such.having another precondition:layer'>
StartLayerSetupTest: <class 'test_such.having another precondition:layer'>:test 0000: should do that not this \(test_such.having another precondition\)
StopLayerSetupTest: <class 'test_such.having another precondition:layer'>:test 0000: should do that not this \(test_such.having another precondition\)
StartLayerTeardownTest: <class 'test_such.having another precondition:layer'>:test 0000: should do that not this \(test_such.having another precondition\)
StopLayerTeardownTest: <class 'test_such.having another precondition:layer'>:test 0000: should do that not this \(test_such.having another precondition\)
StartLayerSetupTest: <class 'test_such.having another precondition:layer'>:test 0001: should do this not that \(test_such.having another precondition\)
StopLayerSetupTest: <class 'test_such.having another precondition:layer'>:test 0001: should do this not that \(test_such.having another precondition\)
StartLayerTeardownTest: <class 'test_such.having another precondition:layer'>:test 0001: should do this not that \(test_such.having another precondition\)
StopLayerTeardownTest: <class 'test_such.having another precondition:layer'>:test 0001: should do this not that \(test_such.having another precondition\)
StartLayerTeardown: <class 'test_such.having another precondition:layer'>
StopLayerTeardown: <class 'test_such.having another precondition:layer'>
StartLayerSetup: <class 'test_such.SomeLayer'>
StopLayerSetup: <class 'test_such.SomeLayer'>
StartLayerSetup: <class 'test_such.having a different precondition:layer'>
StopLayerSetup: <class 'test_such.having a different precondition:layer'>
StartLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0000: should do something else \(test_such.having a different precondition\)
StopLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0000: should do something else \(test_such.having a different precondition\)
StartLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0000: should do something else \(test_such.having a different precondition\)
StopLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0000: should do something else \(test_such.having a different precondition\)
StartLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0001: should have another test \(test_such.having a different precondition\)
StopLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0001: should have another test \(test_such.having a different precondition\)
StartLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0001: should have another test \(test_such.having a different precondition\)
StopLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0001: should have another test \(test_such.having a different precondition\)
StartLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0002: should have access to an external fixture \(test_such.having a different precondition\)
StopLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0002: should have access to an external fixture \(test_such.having a different precondition\)
StartLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0002: should have access to an external fixture \(test_such.having a different precondition\)
StopLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0002: should have access to an external fixture \(test_such.having a different precondition\)
StartLayerSetup: <class 'test_such.having a case inside the external fixture:layer'>
StopLayerSetup: <class 'test_such.having a case inside the external fixture:layer'>
StartLayerSetupTest: <class 'test_such.having a case inside the external fixture:layer'>:test 0000: should still have access to that fixture \(test_such.having a case inside the external fixture\)
StopLayerSetupTest: <class 'test_such.having a case inside the external fixture:layer'>:test 0000: should still have access to that fixture \(test_such.having a case inside the external fixture\)
StartLayerTeardownTest: <class 'test_such.having a case inside the external fixture:layer'>:test 0000: should still have access to that fixture \(test_such.having a case inside the external fixture\)
StopLayerTeardownTest: <class 'test_such.having a case inside the external fixture:layer'>:test 0000: should still have access to that fixture \(test_such.having a case inside the external fixture\)
StartLayerTeardown: <class 'test_such.having a case inside the external fixture:layer'>
StopLayerTeardown: <class 'test_such.having a case inside the external fixture:layer'>
StartLayerTeardown: <class 'test_such.having a different precondition:layer'>
StopLayerTeardown: <class 'test_such.having a different precondition:layer'>
StartLayerTeardown: <class 'test_such.SomeLayer'>
StopLayerTeardown: <class 'test_such.SomeLayer'>
StartLayerTeardown: <class 'test_such.having an expensive fixture:layer'>
StopLayerTeardown: <class 'test_such.having an expensive fixture:layer'>
StartLayerTeardown: <class 'test_such.A system with complex setup:layer'>
StopLayerTeardown: <class 'test_such.A system with complex setup:layer'>\n*$"""  # noqa: E501
        self.assertTestRunOutputMatches(proc, stderr="Ran 9 tests")
        self.assertTestRunOutputMatches(proc, stdout=expected)
        self.assertEqual(proc.poll(), 0, proc.stderr.getvalue())

    def test_simple_layers(self):
        proc = self.runIn(
            "scenario/layers_hooks",
            "--plugin=nose2.plugins.layers",
            "--plugin=layer_hooks_plugin",
            "test_layers_simple",
        )
        if sys.version_info >= (3, 11):
            expected = r"""^StartLayerSetup: <class 'test_layers_simple.Layer1'>
StopLayerSetup: <class 'test_layers_simple.Layer1'>
StartLayerSetupTest: <class 'test_layers_simple.Layer1'>:test_1 \(test_layers_simple.TestSimple.test_1\)
StopLayerSetupTest: <class 'test_layers_simple.Layer1'>:test_1 \(test_layers_simple.TestSimple.test_1\)
StartLayerTeardownTest: <class 'test_layers_simple.Layer1'>:test_1 \(test_layers_simple.TestSimple.test_1\)
StopLayerTeardownTest: <class 'test_layers_simple.Layer1'>:test_1 \(test_layers_simple.TestSimple.test_1\)
StartLayerSetupTest: <class 'test_layers_simple.Layer1'>:test_2 \(test_layers_simple.TestSimple.test_2\)
StopLayerSetupTest: <class 'test_layers_simple.Layer1'>:test_2 \(test_layers_simple.TestSimple.test_2\)
StartLayerTeardownTest: <class 'test_layers_simple.Layer1'>:test_2 \(test_layers_simple.TestSimple.test_2\)
StopLayerTeardownTest: <class 'test_layers_simple.Layer1'>:test_2 \(test_layers_simple.TestSimple.test_2\)
StartLayerTeardown: <class 'test_layers_simple.Layer1'>
StopLayerTeardown: <class 'test_layers_simple.Layer1'>\n*$"""  # noqa: E501
        else:
            expected = r"""^StartLayerSetup: <class 'test_layers_simple.Layer1'>
StopLayerSetup: <class 'test_layers_simple.Layer1'>
StartLayerSetupTest: <class 'test_layers_simple.Layer1'>:test_1 \(test_layers_simple.TestSimple\)
StopLayerSetupTest: <class 'test_layers_simple.Layer1'>:test_1 \(test_layers_simple.TestSimple\)
StartLayerTeardownTest: <class 'test_layers_simple.Layer1'>:test_1 \(test_layers_simple.TestSimple\)
StopLayerTeardownTest: <class 'test_layers_simple.Layer1'>:test_1 \(test_layers_simple.TestSimple\)
StartLayerSetupTest: <class 'test_layers_simple.Layer1'>:test_2 \(test_layers_simple.TestSimple\)
StopLayerSetupTest: <class 'test_layers_simple.Layer1'>:test_2 \(test_layers_simple.TestSimple\)
StartLayerTeardownTest: <class 'test_layers_simple.Layer1'>:test_2 \(test_layers_simple.TestSimple\)
StopLayerTeardownTest: <class 'test_layers_simple.Layer1'>:test_2 \(test_layers_simple.TestSimple\)
StartLayerTeardown: <class 'test_layers_simple.Layer1'>
StopLayerTeardown: <class 'test_layers_simple.Layer1'>\n*$"""  # noqa: E501
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertTestRunOutputMatches(proc, stdout=expected)
        self.assertEqual(proc.poll(), 0, proc.stderr.getvalue())



from nose2.tests._common import FunctionalTestCase, _method_name, windows_ci_skip

_SUFFIX = """\
----------------------------------------------------------------------
Ran 1 test """

Q_TEST_PATTERN = r"(?<!\.)(?<!ok)" + _SUFFIX
MID_TEST_PATTERN = r"\." + "\n" + _SUFFIX
V_TEST_PATTERN = (
    r"test \(__main__\.Test" + _method_name() + r"\) \.\.\. ok" + "\n\n" + _SUFFIX
)


class TestVerbosity(FunctionalTestCase):
    @windows_ci_skip
    def test_no_modifier(self):
        proc = self.runModuleAsMain("scenario/one_test/tests.py")
        self.assertTestRunOutputMatches(proc, stderr=MID_TEST_PATTERN)

    @windows_ci_skip
    def test_one_v(self):
        proc = self.runModuleAsMain("scenario/one_test/tests.py", "-v")
        self.assertTestRunOutputMatches(proc, stderr=V_TEST_PATTERN)

    @windows_ci_skip
    def test_one_q(self):
        proc = self.runModuleAsMain("scenario/one_test/tests.py", "-q")
        self.assertTestRunOutputMatches(proc, stderr=Q_TEST_PATTERN)

    @windows_ci_skip
    def test_one_q_one_v(self):
        proc = self.runModuleAsMain("scenario/one_test/tests.py", "-q", "-v")
        self.assertTestRunOutputMatches(proc, stderr=MID_TEST_PATTERN)

        proc = self.runModuleAsMain("scenario/one_test/tests.py", "-qv")
        self.assertTestRunOutputMatches(proc, stderr=MID_TEST_PATTERN)

    @windows_ci_skip
    def test_one_q_two_vs(self):
        proc = self.runModuleAsMain("scenario/one_test/tests.py", "-q", "-v", "-v")
        self.assertTestRunOutputMatches(proc, stderr=V_TEST_PATTERN)

        proc = self.runModuleAsMain("scenario/one_test/tests.py", "-qvv")
        self.assertTestRunOutputMatches(proc, stderr=V_TEST_PATTERN)

    @windows_ci_skip
    def test_one_v_two_qs(self):
        proc = self.runModuleAsMain("scenario/one_test/tests.py", "-v", "-q", "-q")
        self.assertTestRunOutputMatches(proc, stderr=Q_TEST_PATTERN)

        proc = self.runModuleAsMain("scenario/one_test/tests.py", "-vqq")
        self.assertTestRunOutputMatches(proc, stderr=Q_TEST_PATTERN)

    @windows_ci_skip
    def test_explicit_verbosity(self):
        proc = self.runModuleAsMain("scenario/one_test/tests.py", "--verbosity", "0")
        self.assertTestRunOutputMatches(proc, stderr=Q_TEST_PATTERN)

        proc = self.runModuleAsMain("scenario/one_test/tests.py", "--verbosity", "1")
        self.assertTestRunOutputMatches(proc, stderr=MID_TEST_PATTERN)

        proc = self.runModuleAsMain("scenario/one_test/tests.py", "--verbosity", "2")
        self.assertTestRunOutputMatches(proc, stderr=V_TEST_PATTERN)

        proc = self.runModuleAsMain("scenario/one_test/tests.py", "--verbosity", "-1")
        self.assertTestRunOutputMatches(proc, stderr=Q_TEST_PATTERN)

    @windows_ci_skip
    def test_tweaked_explicit_verbosity(self):
        proc = self.runModuleAsMain(
            "scenario/one_test/tests.py", "--verbosity", "0", "-vv"
        )
        self.assertTestRunOutputMatches(proc, stderr=V_TEST_PATTERN)
        proc = self.runModuleAsMain(
            "scenario/one_test/tests.py", "-vv", "--verbosity", "0"
        )
        self.assertTestRunOutputMatches(proc, stderr=V_TEST_PATTERN)

        proc = self.runModuleAsMain(
            "scenario/one_test/tests.py", "--verbosity", "2", "-q"
        )
        self.assertTestRunOutputMatches(proc, stderr=MID_TEST_PATTERN)
        proc = self.runModuleAsMain(
            "scenario/one_test/tests.py", "-q", "--verbosity", "2", "-q"
        )
        self.assertTestRunOutputMatches(proc, stderr=Q_TEST_PATTERN)



"""
The __init__ file is loaded *before* the testsuite starts running in test
scenarios.
Therefore, even though it would be *great* if we could check that it gets
counted correctly by coverage, it's better to leave it out.
"""



import multiprocessing
import queue
import sys
import threading
import time
import unittest
from multiprocessing import connection

from nose2 import session
from nose2.plugins import buffer
from nose2.plugins.loader import discovery, testcases
from nose2.plugins.mp import MultiProcess, procserver
from nose2.tests._common import Conn, FunctionalTestCase, _method_name, support_file


class TestMpPlugin(FunctionalTestCase):
    def setUp(self):
        super().setUp()
        self.session = session.Session()
        self.plugin = MultiProcess(session=self.session)
        self.plugin.testRunTimeout = 2

    def test_flatten_without_fixtures(self):
        sys.path.append(support_file("scenario/slow"))
        import test_slow as mod

        suite = unittest.TestSuite()
        suite.addTest(mod.TestSlow("test_ok"))
        suite.addTest(mod.TestSlow("test_fail"))
        suite.addTest(mod.TestSlow("test_err"))

        flat = list(self.plugin._flatten(suite))
        self.assertEqual(len(flat), 3)

    def test_flatten_nested_suites(self):
        sys.path.append(support_file("scenario/slow"))
        import test_slow as mod

        suite = unittest.TestSuite()
        suite.addTest(mod.TestSlow("test_ok"))
        suite.addTest(mod.TestSlow("test_fail"))
        suite.addTest(mod.TestSlow("test_err"))

        suite2 = unittest.TestSuite()
        suite2.addTest(suite)

        flat = list(self.plugin._flatten(suite2))
        self.assertEqual(len(flat), 3)

    def test_flatten_respects_module_fixtures(self):
        sys.path.append(support_file("scenario/module_fixtures"))
        import test_mf_testcase as mod

        suite = unittest.TestSuite()
        suite.addTest(mod.Test("test_1"))
        suite.addTest(mod.Test("test_2"))

        flat = list(self.plugin._flatten(suite))
        self.assertEqual(flat, ["test_mf_testcase"])

    def test_flatten_respects_class_fixtures(self):
        sys.path.append(support_file("scenario/class_fixtures"))
        import test_cf_testcase as mod

        suite = unittest.TestSuite()
        suite.addTest(mod.Test("test_1"))
        suite.addTest(mod.Test("test_2"))
        suite.addTest(mod.Test2("test_1"))
        suite.addTest(mod.Test2("test_2"))
        suite.addTest(mod.Test3("test_3"))

        flat = list(self.plugin._flatten(suite))
        self.assertEqual(
            flat,
            [
                "test_cf_testcase.Test2.test_1",
                "test_cf_testcase.Test2.test_2",
                "test_cf_testcase.Test",
                "test_cf_testcase.Test3",
            ],
        )

    def test_conn_prep(self):
        self.plugin.bind_host = None
        (parent_conn, child_conn) = self.plugin._prepConns()
        (parent_pipe, child_pipe) = multiprocessing.Pipe()
        self.assertIsInstance(parent_conn, type(parent_pipe))
        self.assertIsInstance(child_conn, type(child_pipe))

        self.plugin.bind_host = "127.0.0.1"
        self.plugin.bind_port = 0
        (parent_conn, child_conn) = self.plugin._prepConns()
        self.assertIsInstance(parent_conn, connection.Listener)
        self.assertIsInstance(child_conn, tuple)
        self.assertEqual(parent_conn.address, child_conn[:2])

    def test_conn_accept(self):
        (parent_conn, child_conn) = multiprocessing.Pipe()
        self.assertEqual(self.plugin._acceptConns(parent_conn), parent_conn)

        listener = connection.Listener(("127.0.0.1", 0))
        with self.assertRaises(RuntimeError):
            self.plugin._acceptConns(listener)

        def fake_client(address):
            client = connection.Client(address)
            time.sleep(self.plugin.testRunTimeout)
            client.close()

        t = threading.Thread(target=fake_client, args=(listener.address,))
        t.start()
        conn = self.plugin._acceptConns(listener)
        self.assertTrue(hasattr(conn, "send"))
        self.assertTrue(hasattr(conn, "recv"))
        t.join()


class TestProcserver(FunctionalTestCase):
    def setUp(self):
        super().setUp()
        self.session = session.Session()

    def test_dispatch_tests_receive_events(self):
        ssn = {
            "config": self.session.config,
            "verbosity": 1,
            "startDir": support_file("scenario/tests_in_package"),
            "topLevelDir": support_file("scenario/tests_in_package"),
            "logLevel": 100,
            "pluginClasses": [
                discovery.DiscoveryLoader,
                testcases.TestCaseLoader,
                buffer.OutputBufferPlugin,
            ],
        }
        conn = Conn(
            [
                "pkg1.test.test_things.SomeTests.test_ok",
                "pkg1.test.test_things.SomeTests.test_failed",
            ]
        )
        procserver(ssn, conn)

        # check conn calls
        expect = [
            (
                "pkg1.test.test_things.SomeTests.test_ok",
                [
                    ("startTest", {}),
                    ("setTestOutcome", {"outcome": "passed"}),
                    ("testOutcome", {"outcome": "passed"}),
                    ("stopTest", {}),
                ],
            ),
            (
                "pkg1.test.test_things.SomeTests.test_failed",
                [
                    ("startTest", {}),
                    (
                        "setTestOutcome",
                        {
                            "outcome": "failed",
                            "expected": False,
                            "metadata": {
                                "stdout": """\
-------------------- >> begin captured stdout << ---------------------
Hello stdout

--------------------- >> end captured stdout << ----------------------"""
                            },
                        },
                    ),
                    (
                        "testOutcome",
                        {
                            "outcome": "failed",
                            "expected": False,
                            "metadata": {
                                "stdout": """\
-------------------- >> begin captured stdout << ---------------------
Hello stdout

--------------------- >> end captured stdout << ----------------------"""
                            },
                        },
                    ),
                    ("stopTest", {}),
                ],
            ),
        ]
        for val in conn.sent:
            if val is None:
                break
            test, events = val
            exp_test, exp_events = expect.pop(0)
            self.assertEqual(test, exp_test)
            for method, event in events:
                exp_meth, exp_attr = exp_events.pop(0)
                self.assertEqual(method, exp_meth)
                for attr, val in exp_attr.items():
                    self.assertEqual(getattr(event, attr), val)


class MPPluginTestRuns(FunctionalTestCase):
    def test_tests_in_package(self):
        proc = self.runIn(
            "scenario/tests_in_package", "-v", "--plugin=nose2.plugins.mp", "-N=2"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 25 tests")
        self.assertEqual(proc.poll(), 1)

    def test_package_in_lib(self):
        proc = self.runIn(
            "scenario/package_in_lib", "-v", "--plugin=nose2.plugins.mp", "-N=2"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 3 tests")
        self.assertEqual(proc.poll(), 1)

    def test_module_fixtures(self):
        proc = self.runIn(
            "scenario/module_fixtures", "-v", "--plugin=nose2.plugins.mp", "-N=2"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 5 tests")
        self.assertEqual(proc.poll(), 0)

    def test_class_fixtures(self):
        proc = self.runIn(
            "scenario/class_fixtures", "-v", "--plugin=nose2.plugins.mp", "-N=2"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 7 tests")
        self.assertEqual(proc.poll(), 0)

    def test_large_number_of_tests_stresstest(self):
        proc = self.runIn(
            "scenario/many_tests",
            "-v",
            "--plugin=nose2.plugins.mp",
            "--plugin=nose2.plugins.loader.generators",
            "-N=1",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 600 tests")
        self.assertEqual(proc.poll(), 0)

    def test_socket_stresstest(self):
        proc = self.runIn(
            "scenario/many_tests_socket",
            "-v",
            "-c scenario/many_test_socket/nose2.cfg",
            "--plugin=nose2.plugins.mp",
            "--plugin=nose2.plugins.loader.generators",
            "-N=1",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 600 tests")
        self.assertEqual(proc.poll(), 0)

    def test_too_many_procs(self):
        # Just need to run the mp plugin with less tests than
        # processes.
        proc = self.runModuleAsMain(
            "scenario/one_test/tests.py",
            "--log-level=debug",
            "--plugin=nose2.plugins.mp",
            "-N=2",
        )
        ret_vals = queue.Queue()

        def save_return():
            """
            Popen.communicate() blocks.  Use a thread-safe queue
            to return any exceptions.  Ideally, this completes
            and returns None.
            """
            try:
                self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
                self.assertEqual(proc.poll(), 0)
                ret_vals.put(None)
            except Exception as exc:
                ret_vals.put(exc)

        thread = threading.Thread(target=save_return)
        thread.start()

        # 1 minute should be more than sufficient for this
        # little test case.
        try:
            exc = ret_vals.get(True, 60)
        except queue.Empty:
            exc = "MP Test timed out"
            proc.kill()
        self.assertIsNone(exc, str(exc))

    def test_with_output_buffer(self):
        proc = self.runIn(
            "scenario/module_fixtures",
            "-v",
            "--plugin=nose2.plugins.mp",
            "--plugin=nose2.plugins.buffer",
            "-N=2",
            "-B",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 5 tests")
        self.assertEqual(proc.poll(), 0)

    def test_unknown_module(self):
        proc = self.runIn(
            "scenario/module_fixtures",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "-B",
            "does.not.exists.module",
            "does.not.exists.module2",
        )
        expected_results = (
            r"does\.not\.exists\.module2 (\S+) \.\.\. ERROR\n"
            r"does\.not\.exists\.module (\S+) \.\.\. ERROR"
        )
        self.assertTestRunOutputMatches(proc, stderr=expected_results)
        self.assertEqual(proc.poll(), 1)


class MPTestClassSupport(FunctionalTestCase):
    def test_testclass_discover(self):
        proc = self.runIn(
            "scenario/test_classes_mp", "-v", "--plugin=nose2.plugins.mp", "-N=2"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 13 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_by_module(self):
        proc = self.runIn(
            "scenario/test_classes_mp",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_classes_mp",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 8 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_by_class(self):
        proc = self.runIn(
            "scenario/test_classes_mp",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_classes_mp.Test",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 8 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_parameters(self):
        proc = self.runIn(
            "scenario/test_classes_mp",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_classes_mp.Test.test_params",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_generators(self):
        proc = self.runIn(
            "scenario/test_classes_mp",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_classes_mp.Test.test_gen",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 5 tests")
        self.assertEqual(proc.poll(), 0)


class MPClassFixturesSupport(FunctionalTestCase):
    def test_testcase_class_fixtures(self):
        proc = self.runIn(
            "scenario/class_fixtures", "-v", "test_cf_testcase.Test.test_1"
        )
        # main process runs selected tests
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertEqual(proc.poll(), 0)

    def test_testcase_class_fixtures_mp(self):
        proc = self.runIn(
            "scenario/class_fixtures",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_cf_testcase.Test.test_1",
        )
        # XXX mp plugin runs the entire class if a class fixture is detected
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testcase_class_fixtures_report_mp(self):
        proc = self.runIn(
            "scenario/class_fixtures",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_cf_testcase.Test.test_1",
        )
        # report should show correct names for all tests
        self.assertTestRunOutputMatches(
            proc,
            stderr=r"test_1 \(test_cf_testcase.Test"
            + _method_name("test_1")
            + r"\) ... ok",
        )
        self.assertTestRunOutputMatches(
            proc,
            stderr=r"test_2 \(test_cf_testcase.Test"
            + _method_name("test_2")
            + r"\) ... ok",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_class_fixtures_and_parameters(self):
        proc = self.runIn(
            "scenario/test_classes_mp", "-v", "test_fixtures_mp.Test.test_params"
        )
        # main process runs selected tests
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_class_fixtures_and_parameters_mp(self):
        proc = self.runIn(
            "scenario/test_classes_mp",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_fixtures_mp.Test.test_params",
        )
        # XXX mp plugin runs the entire class if a class fixture is detected
        self.assertTestRunOutputMatches(proc, stderr="Ran 5 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_class_fixtures_and_generators(self):
        proc = self.runIn(
            "scenario/test_classes_mp", "-v", "test_fixtures_mp.Test.test_gen"
        )
        # main process runs selected tests
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_class_fixtures_and_generators_mp(self):
        proc = self.runIn(
            "scenario/test_classes_mp",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_fixtures_mp.Test.test_gen",
        )
        # XXX mp plugin runs the entire class if a class fixture is detected
        self.assertTestRunOutputMatches(proc, stderr="Ran 5 tests")
        self.assertEqual(proc.poll(), 0)


class MPModuleFixturesSupport(FunctionalTestCase):
    def test_testcase_module_fixtures(self):
        proc = self.runIn(
            "scenario/module_fixtures", "-v", "test_mf_testcase.Test.test_1"
        )
        # main process runs selected tests
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertEqual(proc.poll(), 0)

    def test_testcase_module_fixtures_mp(self):
        proc = self.runIn(
            "scenario/module_fixtures",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_mf_testcase.Test.test_1",
        )
        # XXX mp plugin runs the entire module if a module fixture is detected
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testcase_module_fixtures_report_mp(self):
        proc = self.runIn(
            "scenario/module_fixtures",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_mf_testcase.Test.test_1",
        )
        # report should show correct names for all tests
        self.assertTestRunOutputMatches(
            proc,
            stderr=r"test_1 \(test_mf_testcase.Test"
            + _method_name("test_1")
            + r"\) ... ok",
        )
        self.assertTestRunOutputMatches(
            proc,
            stderr=r"test_2 \(test_mf_testcase.Test"
            + _method_name("test_2")
            + r"\) ... ok",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertEqual(proc.poll(), 0)



from nose2 import events, loader, session, util
from nose2.plugins.loader import parameters, testcases
from nose2.tests._common import TestCase
from nose2.tools import cartesian_params, params


class TestParams(TestCase):
    tags = ["unit"]

    def setUp(self):
        self.session = session.Session()
        self.loader = loader.PluggableTestLoader(self.session)
        self.plugin = parameters.Parameters(session=self.session)
        # need testcase loader to make the initial response to load from module
        self.tcl = testcases.TestCaseLoader(session=self.session)

    def test_ignores_ordinary_functions(self):
        class Mod:
            pass

        def test():
            pass

        m = Mod()
        m.test = test
        event = events.LoadFromModuleEvent(self.loader, m)
        self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(len(event.extraTests), 0)

    def test_can_load_tests_from_parameterized_by_params_functions(self):
        class Mod:
            __name__ = "themod"

        def check(x):
            assert x == 1

        @params(1, 2)
        def test(a):
            check(a)

        m = Mod()
        m.test = test
        test.__module__ = m.__name__
        event = events.LoadFromModuleEvent(self.loader, m)
        self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(len(event.extraTests), 2)
        # check that test names are sensible
        self.assertEqual(util.test_name(event.extraTests[0]), "themod.test:1")
        self.assertEqual(util.test_name(event.extraTests[1]), "themod.test:2")

    def test_can_load_tests_from_parameterized_by_cartesian_params_functions(self):
        class Mod:
            __name__ = "themod"

        def check(x, y):
            assert x == y

        @cartesian_params((1, 2), (2, 3))
        def test(a, b):
            check(a, b)

        m = Mod()
        m.test = test
        test.__module__ = m.__name__
        event = events.LoadFromModuleEvent(self.loader, m)
        self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(len(event.extraTests), 4)
        # check that test names are sensible
        self.assertEqual(util.test_name(event.extraTests[0]), "themod.test:1")
        self.assertEqual(util.test_name(event.extraTests[1]), "themod.test:2")
        self.assertEqual(util.test_name(event.extraTests[2]), "themod.test:3")
        self.assertEqual(util.test_name(event.extraTests[3]), "themod.test:4")

    def test_can_load_tests_from_parameterized_by_params_methods(self):
        class Mod:
            __name__ = "themod"

        class Test(TestCase):
            @params(1, 2)
            def test(self, a):
                assert a == 1

        m = Mod()
        m.Test = Test
        Test.__module__ = m.__name__
        event = events.LoadFromModuleEvent(self.loader, m)
        self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(len(event.extraTests), 1)
        self.assertEqual(len(event.extraTests[0]._tests), 2)
        # check that test names are sensible
        t1 = util.test_name(event.extraTests[0]._tests[0], qualname=False)
        self.assertEqual(t1, "themod.Test.test:1")
        t2 = util.test_name(event.extraTests[0]._tests[1], qualname=False)
        self.assertEqual(t2, "themod.Test.test:2")

    def test_can_load_tests_from_parameterized_by_cartesian_params_methods(self):
        class Mod:
            __name__ = "themod"

        class Test(TestCase):
            @cartesian_params((1, 2), (2, 3))
            def test(self, a, b):
                assert a == b

        m = Mod()
        m.Test = Test
        Test.__module__ = m.__name__
        event = events.LoadFromModuleEvent(self.loader, m)
        self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(len(event.extraTests), 1)
        self.assertEqual(len(event.extraTests[0]._tests), 4)
        # check that test names are sensible
        self.assertEqual(
            util.test_name(event.extraTests[0]._tests[0]), "themod.Test.test:1"
        )
        self.assertEqual(
            util.test_name(event.extraTests[0]._tests[1]), "themod.Test.test:2"
        )
        self.assertEqual(
            util.test_name(event.extraTests[0]._tests[2]), "themod.Test.test:3"
        )
        self.assertEqual(
            util.test_name(event.extraTests[0]._tests[3]), "themod.Test.test:4"
        )

    def test_params_creates_params_for_function(self):
        @params((1, 2), ("a", "b"))
        def test(a, b):
            assert a == b

        self.assertTupleEqual(tuple(test.paramList), ((1, 2), ("a", "b")))

    def test_cartesian_params_creates_cartesian_product_of_params_for_function(self):
        @cartesian_params((1, 2), ("a", "b"))
        def test(a, b):
            assert a == b

        self.assertTupleEqual(
            tuple(test.paramList), ((1, "a"), (1, "b"), (2, "a"), (2, "b"))
        )

    def test_params_creates_params_for_method(self):
        class Test(TestCase):
            @params((1, 2), ("a", "b"))
            def test(self, a, b):
                assert a == b

        self.assertTupleEqual(tuple(Test.test.paramList), ((1, 2), ("a", "b")))

    def test_cartesian_params_creates_cartesian_product_of_params_for_method(self):
        class Test(TestCase):
            @cartesian_params((1, 2), ("a", "b"))
            def test(self, a, b):
                assert a == b

        self.assertTupleEqual(
            tuple(Test.test.paramList), ((1, "a"), (1, "b"), (2, "a"), (2, "b"))
        )



import io
import sys

from nose2 import events, result, session, util
from nose2.plugins import buffer
from nose2.tests._common import TestCase


class TestBufferPlugin(TestCase):
    tags = ["unit"]

    def setUp(self):
        self.session = session.Session()
        self.result = result.PluggableTestResult(self.session)
        self.plugin = buffer.OutputBufferPlugin(session=self.session)
        self.plugin.register()

        class Test(TestCase):
            printed_nonascii_str = util.safe_decode("test 日本").encode("utf-8")
            printed_unicode = "hello"

            def test_out(self):
                print("hello")
                raise {}["oops"]

            def test_err(self):
                print("goodbye", file=sys.stderr)

            def test_mixed_unicode_and_nonascii_str(self):
                print(self.printed_nonascii_str)
                print(self.printed_unicode)
                print(self.printed_nonascii_str, file=sys.stderr)
                print(self.printed_unicode, file=sys.stderr)
                raise {}["oops"]

        self.case = Test

        class Watcher(events.Plugin):
            def __init__(self):
                self.events = []

            def testOutcome(self, event):
                self.events.append(event)

        self.watcher = Watcher(session=self.session)
        self.watcher.register()

    def test_captures_stdout(self):
        out = sys.stdout
        buf = io.StringIO()
        sys.stdout = buf
        try:
            test = self.case("test_out")
            test(self.result)
            assert "hello" not in buf.getvalue()
            assert "hello" in self.watcher.events[0].metadata["stdout"]
        finally:
            sys.stdout = out

    def test_captures_stderr_when_configured(self):
        self.plugin.captureStderr = True
        err = sys.stderr
        buf = io.StringIO()
        sys.stderr = buf
        try:
            test = self.case("test_err")
            test(self.result)
            assert "goodbye" not in buf.getvalue()
            assert "goodbye" in self.watcher.events[0].metadata["stderr"]
        finally:
            sys.stderr = err

    def test_does_not_crash_with_mixed_unicode_and_nonascii_str(self):
        self.plugin.captureStderr = True
        test = self.case("test_mixed_unicode_and_nonascii_str")
        test(self.result)
        evt = events.OutcomeDetailEvent(self.watcher.events[0])
        self.session.hooks.outcomeDetail(evt)
        extraDetail = "".join(evt.extraDetail)
        for string in [
            repr(self.case.printed_nonascii_str),
            self.case.printed_unicode,
        ]:
            assert string in extraDetail, "Output not found in error message"

    def test_decorates_outcome_detail(self):
        test = self.case("test_out")
        test(self.result)
        evt = events.OutcomeDetailEvent(self.watcher.events[0])
        self.session.hooks.outcomeDetail(evt)
        assert "hello" in "".join(evt.extraDetail)



from nose2.tests._common import FunctionalTestCase, _method_name


class TestLayers(FunctionalTestCase):
    def test_runs_layer_fixtures(self):
        proc = self.runIn("scenario/layers", "-v", "--plugin=nose2.plugins.layers")
        self.assertTestRunOutputMatches(proc, stderr="Ran 8 tests")
        self.assertEqual(proc.poll(), 0)

    def test_scenario_fails_without_plugin(self):
        proc = self.runIn("scenario/layers", "-v")
        self.assertTestRunOutputMatches(proc, stderr="Ran 8 tests")
        self.assertTestRunOutputMatches(proc, stderr=r"FAILED \(failures=7\)")
        self.assertEqual(proc.poll(), 1)

    def test_methods_run_once_per_class(self):
        proc = self.runIn(
            "scenario/layers_with_inheritance", "-v", "--plugin=nose2.plugins.layers"
        )

        expected = (
            "^"
            "L1 setUp\n"
            "L2 setUp\n"
            "L1 testSetUp\n"
            "L2 testSetUp\n"
            "Run test1\n"
            "L2 testTearDown\n"
            "L1 testTearDown\n"
            "L1 testSetUp\n"
            "L2 testSetUp\n"
            "Run test2\n"
            "L2 testTearDown\n"
            "L1 testTearDown\n"
            "L1 tearDown\n"
            "$"
        )
        self.assertTestRunOutputMatches(proc, stdout=expected)
        self.assertEqual(proc.poll(), 0)

    def test_layer_reporter_output(self):
        proc = self.runIn(
            "scenario/layers", "-v", "--plugin=nose2.plugins.layers", "--layer-reporter"
        )
        expect = (
            r"""test \(test_layers.NoLayer"""
            + _method_name()
            + r"""\) ... ok
Base
  test \(test_layers.Outer"""
            + _method_name()
            + r"""\) ... ok
  LayerD
    test \(test_layers.InnerD"""
            + _method_name()
            + r"""\) test with docstring ... ok
  LayerA
    test \(test_layers.InnerA"""
            + _method_name()
            + r"""\) ... ok
  LayerB
    LayerB_1
      test \(test_layers.InnerB_1"""
            + _method_name()
            + r"""\) ... ok
    LayerC
      test \(test_layers.InnerC"""
            + _method_name()
            + r"""\) ... ok
      test2 \(test_layers.InnerC"""
            + _method_name("test2")
            + r"""\) ... ok
    LayerA_1
      test \(test_layers.InnerA_1"""
            + _method_name()
            + r"""\) ... ok"""
        ).split("\n")
        self.assertTestRunOutputMatches(proc, stderr="Ran 8 tests")
        for line in expect:
            self.assertTestRunOutputMatches(proc, stderr=line)
        self.assertEqual(proc.poll(), 0)

    def test_layer_reporter_error_output(self):
        proc = self.runIn(
            "scenario/layers_with_errors",
            "--plugin=nose2.plugins.layers",
            "--layer-reporter",
        )
        expect = [
            r"ERROR: fixture with a value test_err \(test_layers_with_errors.Test"
            + _method_name("test_err")
            + r"\)",
            "ERROR: A test scenario with errors should check for an attribute "
            "that does not exist and raise an error",
            r"FAIL: fixture with a value test_fail \(test_layers_with_errors.Test"
            + _method_name("test_fail")
            + r"\)",
            "FAIL: A test scenario with errors should check that value == 2 "
            "and fail",
        ]
        for line in expect:
            self.assertTestRunOutputMatches(proc, stderr=line)
        self.assertEqual(proc.poll(), 1)

    def test_layers_and_attributes(self):
        proc = self.runIn(
            "scenario/layers_and_attributes",
            "-v",
            "--plugin=nose2.plugins.attrib",
            "--plugin=nose2.plugins.layers",
            "-A",
            "a=1",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertEqual(proc.poll(), 0)

    def test_teardown_fail(self):
        proc = self.runIn(
            "scenario/layers_with_errors",
            "--plugin=nose2.plugins.layers",
            "-v",
            "test_layer_teardown_fail",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test in")
        self.assertTestRunOutputMatches(proc, stderr="ERROR: LayerSuite")
        self.assertTestRunOutputMatches(proc, stderr="FAIL")
        self.assertTestRunOutputMatches(proc, stderr="Bad Error in Layer testTearDown")

    def test_setup_fail(self):
        proc = self.runIn(
            "scenario/layers_with_errors",
            "--plugin=nose2.plugins.layers",
            "-v",
            "test_layer_setup_fail",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 0 tests in")
        self.assertTestRunOutputMatches(proc, stderr="ERROR: LayerSuite")
        self.assertTestRunOutputMatches(proc, stderr="FAIL")
        self.assertTestRunOutputMatches(proc, stderr="Bad Error in Layer setUp!")

    def test_layers_and_non_layers(self):
        proc = self.runIn(
            "scenario/", "layers_and_non_layers", "-v", "--plugin=nose2.plugins.layers"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 12 tests in")
        self.assertTestRunOutputMatches(proc, stderr="OK")
        self.assertEqual(proc.poll(), 0)



import configparser
import sys

from nose2 import session
from nose2.plugins import mp
from nose2.tests._common import Conn, TestCase


class TestMPPlugin(TestCase):
    def setUp(self):
        self.session = session.Session()
        self.plugin = mp.MultiProcess(session=self.session)

    def test_gentests(self):
        conn = Conn([1, 2, 3])
        res = []
        for x in mp.gentests(conn):
            res.append(x)
        self.assertEqual(res, [1, 2, 3])

    def test_recording_plugin_interface(self):
        rpi = mp.RecordingPluginInterface()
        # this one should record
        rpi.setTestOutcome(None)
        # none of these should record
        rpi.getTestCaseNames(None)
        rpi.startSubprocess(None)
        rpi.stopSubprocess(None)
        rpi.registerInSubprocess(None)
        rpi.loadTestsFromModule(None)
        rpi.loadTestsFromTestCase(None)
        rpi.moduleLoadedSuite(None)
        rpi.getTestMethodNames(None)
        self.assertEqual(rpi.flush(), [("setTestOutcome", None)])

    def test_address(self):
        platform = sys.platform
        try:
            sys.platform = "linux"
            host = "1.2.3.4"
            port = 245
            self.plugin.setAddress(host)
            self.assertEqual((self.plugin.bind_host, self.plugin.bind_port), (host, 0))
            self.plugin.setAddress("%s:%i" % (host, port))
            self.assertEqual(
                (self.plugin.bind_host, self.plugin.bind_port), (host, port)
            )
            self.plugin.setAddress(None)
            self.assertEqual((self.plugin.bind_host, self.plugin.bind_port), (None, 0))
            sys.platform = "win32"
            self.plugin.setAddress(host)
            self.assertEqual((self.plugin.bind_host, self.plugin.bind_port), (host, 0))
            self.plugin.setAddress("%s:%i" % (host, port))
            self.assertEqual(
                (self.plugin.bind_host, self.plugin.bind_port), (host, port)
            )
            self.plugin.setAddress(None)
            self.assertEqual(
                (self.plugin.bind_host, self.plugin.bind_port), ("127.116.157.163", 0)
            )
        finally:
            sys.platform = platform

    def test_session_import(self):
        config = configparser.ConfigParser()
        config.add_section(mp.MultiProcess.configSection)
        export_session = {
            "config": config,
            "verbosity": None,
            "startDir": "",
            "topLevelDir": "",
            "pluginClasses": [mp.MultiProcess],
        }
        import logging

        session = mp.import_session(logging.root, export_session)
        self.assertIn("registerInSubprocess", session.hooks.methods)
        self.assertIn("startSubprocess", session.hooks.methods)
        self.assertIn("stopSubprocess", session.hooks.methods)
        pass



import re

from nose2.tests._common import FunctionalTestCase


class LogCaptureFunctionalTest(FunctionalTestCase):
    def test_package_in_lib(self):
        match = re.compile(">> begin captured logging <<")
        self.assertTestRunOutputMatches(
            self.runIn("scenario/package_in_lib", "--log-capture"), stderr=match
        )

    def test_logging_keeps_copies_of_mutable_objects(self):
        proc = self.runIn(
            "scenario/logging",
            "-v",
            "--log-capture",
            "logging_keeps_copies_of_mutable_objects",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test in")
        self.assertTestRunOutputMatches(proc, stderr="FAILED")
        self.assertTestRunOutputMatches(proc, stderr="foo: {}")

    def test_logging_config_interpolation(self):
        proc = self.runIn("scenario/logging_config", "-v", "logging_config")
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test in")
        self.assertTestRunOutputMatches(proc, stderr="FAILED")
        self.assertTestRunOutputMatches(
            proc, stderr=r"\[logging_config\] \[DEBUG\] foo"
        )
        self.assertTestRunOutputMatches(proc, stderr=r"\[logging_config\] \[INFO\] bar")



import os
import os.path
import platform
import sys
import unittest

from nose2.tests._common import FunctionalTestCase, support_file, windows_ci_skip


class TestCoverage(FunctionalTestCase):
    def setUp(self):
        super().setUp()
        try:
            import coverage  # noqa: F401
        except ImportError:
            self.skipTest("coverage required")

    def assertProcOutputPattern(
        self, proc, libname, stats, total_stats=None, assert_exit_status=0
    ):
        """
        - proc: a popen proc to check output on
        - libname: the name of the covered library, to build the output pattern
        - stats: the output format for stats info
        - total_stats: optional, distinct stats format for the TOTAL line
                       defaults to be the same as stats
        """
        if total_stats is None:
            total_stats = stats

        expected = os.path.join(libname, "mod1(.py)?")
        expected = expected.replace("\\", r"\\")
        expected = expected + stats

        stdout, stderr = proc.communicate()

        if assert_exit_status is not None:
            self.assertEqual(assert_exit_status, proc.poll())

        self.assertTestRunOutputMatches(proc, stderr=expected)
        self.assertTestRunOutputMatches(proc, stderr=r"TOTAL\s+" + total_stats)

    def test_run(self):
        # ensure there is no .coverage file at the start of test
        reportfile = support_file("scenario/test_with_module/.coverage")
        try:
            os.remove(reportfile)
        except OSError:
            pass

        proc = self.runIn(
            "scenario/test_with_module", "-v", "--with-coverage", "--coverage=lib/"
        )
        self.assertProcOutputPattern(proc, "lib", r"\s+8\s+5\s+38%")
        self.assertTrue(os.path.exists(reportfile))

    def test_run_coverage_configs(self):
        STATS = r"\s+8\s+5\s+38%\s+1, 7-10"
        TOTAL_STATS = r"\s+8\s+5\s+38%\s"

        proc = self.runIn(
            "scenario/test_coverage_config/coveragerc",
            "-v",
            "--with-coverage",
            "--coverage=covered_lib_coveragerc/",
        )
        self.assertProcOutputPattern(
            proc, "covered_lib_coveragerc", STATS, total_stats=TOTAL_STATS
        )

    # unclear if this failure is a problem with the test context (coverage run
    # of nose2 with faked subprocesses) or if it's a bug in the coverage plugin
    # or mp plugin on that platform
    @unittest.skipIf(
        platform.system() == "Darwin" and sys.version_info >= (3, 8),
        "FIXME: this test fails on modern pythons on macos",
    )
    @windows_ci_skip
    def test_run_with_mp(self):
        # this test needs to be done with nose2 config because (as of 2019-12)
        # multiprocessing does not allow each test process to pick up on
        # command line arguments

        # run with 4 processes -- this will fail if `coverage` isn't running in
        # a "parallel" mode (with a "data suffix" set and combining results for
        # reporting)
        proc = self.runIn(
            "scenario/test_coverage_config/nose2cfg",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N",
            "4",
        )
        self.assertProcOutputPattern(
            proc,
            "covered_lib_nose2cfg",
            r"\s+8\s+5\s+38%\s+1, 7-10",
            total_stats=r"\s+8\s+5\s+38%",
        )

    # FIXME: figure out why this fails and remove @skip
    @unittest.skip("fails in testsuite but passes in real-world conditions")
    def test_measures_imports(self):
        proc = self.runIn(
            "scenario/coverage_of_imports",
            "-v",
            "--with-coverage",
            "--coverage=lib20171102/",
        )
        self.assertProcOutputPattern(proc, "lib20171102", stats=r"\s+3\s+0\s+100%")

    def test_run_coverage_fail_under(self):
        STATS = r"\s+8\s+5\s+38%\s+1, 7-10"
        TOTAL_STATS = r"\s+8\s+5\s+38%\s"

        proc = self.runIn(
            "scenario/coverage_config_fail_under",
            "-v",
            "--with-coverage",
            "--coverage=covered_lib/",
        )
        self.assertProcOutputPattern(
            proc, "covered_lib", STATS, total_stats=TOTAL_STATS, assert_exit_status=1
        )

    def test_run_coverage_fail_under2(self):
        """Check with coverage settings in config, not CLI"""
        STATS = r"\s+8\s+5\s+38%\s+1, 7-10"
        TOTAL_STATS = r"\s+8\s+5\s+38%\s"

        proc = self.runIn("scenario/coverage_config_fail_under2", "-v")
        self.assertProcOutputPattern(
            proc,
            "part_covered_lib",
            STATS,
            total_stats=TOTAL_STATS,
            assert_exit_status=1,
        )



TRUE_VALS = {"1", "t", "true", "on", "yes", "y"}
__unittest = True


class Config:

    """Configuration for a plugin or other entities.

    Encapsulates configuration for a single plugin or other element.
    Corresponds to a :class:`ConfigParser.Section` but provides an
    extended interface for extracting items as a certain type.

    """

    def __init__(self, items):
        self._items = items
        self._mvd = {}
        for k, v in items:
            self._mvd.setdefault(k, []).append(v)

    def __getitem__(self, key):
        return self._mvd[key]

    def as_bool(self, key, default=None):
        """Get key value as boolean

        1, t, true, on, yes and y (case insensitive) are accepted as ``True``
        values. All other values are ``False``.

        """
        try:
            val = self._mvd[key][0].strip()
        except KeyError:
            return default
        except IndexError:
            # setting = -> False
            return False
        return val.lower() in TRUE_VALS

    def as_int(self, key, default=None):
        """Get key value as integer"""
        return self._cast(key, int, default)

    def as_float(self, key, default=None):
        """Get key value as float"""
        return self._cast(key, float, default)

    def as_str(self, key, default=None):
        """Get key value as str"""
        return self._cast(key, str, default)

    def as_list(self, key, default=None):
        """Get key value as list.

        The value is split into lines and returned as a list. Lines
        are stripped of whitespace, and lines beginning with # are
        skipped.

        """
        lines = []
        try:
            vlist = self[key]
        except KeyError:
            return default
        for val in vlist:
            lines.extend(
                line.strip()
                for line in val.splitlines()
                if line.strip() and not line.strip().startswith("#")
            )
        return lines

    def get(self, key, default=None):
        """Get key value"""
        return self.as_str(key, default)

    def _cast(self, key, type_, default):
        try:
            return type_(self._mvd[key][0].strip())
        except (KeyError, IndexError):
            return default



"""
Use this plugin to activate coverage report.

To use this plugin, you need to install ``nose2[coverage_plugin]``. e.g.

::

    $ pip install nose2[coverage_plugin]>=0.6.5


Then, you can enable coverage reporting with :

::

    $ nose2 --with-coverage

Or with this lines in ``unittest.cfg`` :

::

    [coverage]
    always-on = True


You can further specify coverage behaviors with a ``.coveragerc`` file, as
specified by `Coverage Config
<http://coverage.readthedocs.io/en/latest/config.html>`_.
However, when doing so you should also be aware of
`Differences From coverage`_.
"""

import io
import logging

from nose2.events import Plugin

log = logging.getLogger(__name__)


class Coverage(Plugin):
    configSection = "coverage"
    commandLineSwitch = ("C", "with-coverage", "Turn on coverage reporting")
    _mpmode = False
    _subprocess = False

    def __init__(self):
        """Get our config and add our command line arguments."""
        # tracking var for any decision which marks the entire run as failed
        self.decided_failure = False
        # buffer for error output data
        self.error_output_buffer = io.StringIO()

        self.covSource = self.config.as_list("coverage", []) or ["."]
        self.covReport = self.config.as_list("coverage-report", []) or ["term"]
        self.covConfig = (
            self.config.as_str("coverage-config", "").strip() or ".coveragerc"
        )

        group = self.session.pluginargs
        group.add_argument(
            "--coverage",
            action="append",
            default=[],
            metavar="PATH",
            dest="coverage_source",
            help="Measure coverage for filesystem path (multi-allowed)",
        )
        group.add_argument(
            "--coverage-report",
            action="append",
            default=[],
            metavar="TYPE",
            choices=["term", "term-missing", "annotate", "html", "xml"],
            dest="coverage_report",
            help="Generate selected reports, available types:"
            " term, term-missing, annotate, html, xml (multi-allowed)",
        )
        group.add_argument(
            "--coverage-config",
            action="store",
            default="",
            metavar="FILE",
            dest="coverage_config",
            help="Config file for coverage, default: .coveragerc",
        )
        self.covController = None

    def registerInSubprocess(self, event):
        event.pluginClasses.append(self.__class__)
        self._mpmode = True

    def handleArgs(self, event):
        """Get our options in order command line, config file, hard coded."""
        self.covSource = event.args.coverage_source or self.covSource
        self.covReport = event.args.coverage_report or self.covReport
        self.covConfig = event.args.coverage_config or self.covConfig

    def createTests(self, event):
        """Start coverage early to catch imported modules.

        Only called if active so, safe to just start without checking flags"""
        if event.handled:
            log.error(
                "createTests already handled -- "
                "coverage reporting will be inaccurate"
            )
        else:
            log.debug("createTests not already handled. coverage should work")

        self._start_coverage()

    def beforeSummaryReport(self, event):
        """Only called if active so stop coverage and produce reports."""
        if self.covController:
            self.covController.stop()

            # write to .coverage file
            # do this explicitly (instead of passing auto_data=True to
            # Coverage constructor) in order to not load an existing .coverage
            # this better imitates the behavior of invoking `coverage` from the
            # command-line, which sets `Coverage._auto_save` (triggers atexit
            # saving to this file), but not `Coverage._auto_load`
            # requesting a better fix in nedbat/coveragepy#34
            self.covController.save()

            if self._mpmode:
                self.covController.combine(strict=True)

            percent_coverage = None

            if "term" in self.covReport or "term-missing" in self.covReport:
                # only pass `show_missing` if "term-missing" was given
                # otherwise, just allow coverage to load show_missing from
                # config
                kwargs = {}
                if "term-missing" in self.covReport:
                    kwargs["show_missing"] = True
                percent_coverage = self.covController.report(
                    file=self.error_output_buffer, **kwargs
                )
            if "annotate" in self.covReport:
                percent_coverage = self.covController.annotate()
            if "html" in self.covReport:
                percent_coverage = self.covController.html_report()
            if "xml" in self.covReport:
                percent_coverage = self.covController.xml_report()

            fail_under = self.covController.get_option("report:fail_under")
            if (
                fail_under is not None
                and percent_coverage is not None
                and fail_under > percent_coverage
            ):
                self.decided_failure = True

    def startSubprocess(self, event):
        self._mpmode = True
        self._subprocess = True
        self._start_coverage()

    def stopSubprocess(self, event):
        self.covController.stop()
        self.covController.save()

    def wasSuccessful(self, event):
        """Mark full test run as successful or unsuccessful"""
        if self.decided_failure:
            event.success = False

    def afterSummaryReport(self, event):
        """Reporting data is collected, failure status determined and set.
        Now print any buffered error output saved from beforeSummaryReport"""
        print(self.error_output_buffer.getvalue(), file=event.stream)

    def _start_coverage(self):
        try:
            import coverage
        except ImportError:
            print(
                'Warning: you need to install "coverage_plugin" '
                "extra requirements to use this plugin. "
                "e.g. `pip install nose2[coverage_plugin]`"
            )
            return

        self.covController = coverage.Coverage(
            source=self.covSource, config_file=self.covConfig, data_suffix=self._mpmode
        )
        # Call erase() to remove old files. This is important in multiprocess
        # mode, where per-process coverage files are unlikely to be
        # overwritten.
        self.covController.erase()
        # start immediately (don't wait until startTestRun) so that coverage
        # will pick up on things which happen at import time
        self.covController.start()



import sys

# This unused import is not very elegant, but it allows eggdiscovery to be found in
# Travis (or when run with PYTHONPATH=.)
import nose2.plugins.loader.eggdiscovery  # noqa: F401
from nose2.tests._common import FunctionalTestCase, support_file

try:
    import pkg_resources

except ImportError:
    pkg_resources = None

else:

    class EggDiscoveryFunctionalTest(FunctionalTestCase):
        def setUp(self):
            for m in [m for m in sys.modules if m.startswith("pkgegg")]:
                del sys.modules[m]
            self.egg_path = support_file(
                "scenario/tests_in_zipped_eggs/pkgegg-0.0.0-py2.7.egg"
            )
            sys.path.append(self.egg_path)

        def tearDown(self):
            if self.egg_path in sys.path:
                sys.path.remove(self.egg_path)
            for m in [m for m in sys.modules if m.startswith("pkgegg")]:
                del sys.modules[m]

        def test_non_egg_discoverer_does_not_fail_when_looking_in_egg(self):
            proc = self.runIn("scenario/tests_in_zipped_eggs", "-v", "pkgegg")
            self.assertTestRunOutputMatches(proc, stderr="Ran 0 tests in")

        def test_can_discover_test_modules_in_zipped_eggs(self):
            proc = self.runIn(
                "scenario/tests_in_zipped_eggs",
                "-v",
                "--plugin=nose2.plugins.loader.eggdiscovery",
                "pkgegg",
            )
            self.assertTestRunOutputMatches(
                proc, stderr=r"FAILED \(failures=5, errors=1, skipped=1\)"
            )

        def test_eggdiscovery_failure_does_not_exist(self):
            proc = self.runIn(
                "scenario",
                "-v",
                "--plugin=nose2.plugins.loader.eggdiscovery",
                "--exclude-plugin=nose2.plugins.loader.discovery",
                "-s",
                "tests_in_zipped_eggs_BAD",
            )
            self.assertTestRunOutputMatches(
                proc, stderr="tests_in_zipped_eggs_BAD does not exist"
            )

    class UnzippedEggDiscoveryFunctionalTest(FunctionalTestCase):
        def setUp(self):
            for m in [m for m in sys.modules if m.startswith("pkgegg")]:
                del sys.modules[m]
            self.egg_path = support_file(
                "scenario/tests_in_unzipped_eggs/pkgunegg-0.0.0-py2.7.egg"
            )
            sys.path.append(self.egg_path)

        def tearDown(self):
            if self.egg_path in sys.path:
                sys.path.remove(self.egg_path)
            for m in [m for m in sys.modules if m.startswith("pkgunegg")]:
                del sys.modules[m]

        def test_eggdiscovery_ignores_unzipped_eggs(self):
            proc = self.runIn(
                "scenario/tests_in_unzipped_eggs",
                "-v",
                "--plugin=nose2.plugins.loader.eggdiscovery",
                "pkgunegg",
            )
            self.assertTestRunOutputMatches(
                proc, stderr=r"FAILED \(failures=5, errors=1, skipped=1\)"
            )



from nose2.tools.decorators import with_setup, with_teardown

setup_performed = False
teardown_performed = False


def setup():
    global setup_performed
    setup_performed = True


def teardown():
    global teardown_performed
    teardown_performed = True


@with_setup(setup)
def test_with_setup():
    assert setup_performed, "Setup not performed."


@with_teardown(teardown)
def test_with_teardown():
    pass


def test_teardown_ran():
    assert teardown_performed, "Teardown not performed."



import logging

from nose2.tools import such

log = logging.getLogger(__name__)


with such.A("foo") as it:
    it.upper_run = 0
    it.lower_run = 0

    @it.has_test_setup
    def upper_test_setup(case):
        log.error("foo::setUp")
        it.upper_run += 1

    @it.should("run upper setups")
    def test_run_upper_setups(case):
        case.assertEqual(it.upper_run, 1)
        case.assertEqual(it.lower_run, 0)

    with it.having("some bar"):

        @it.has_test_setup
        def lower_test_setup(case):
            log.error("foo some bar::setUp")
            it.lower_run += 1

        @it.should("run all setups")
        def test_run_all_setups(case):
            case.assertEqual(it.upper_run, 2)
            case.assertEqual(it.lower_run, 1)

        @it.should("run all setups again")
        def test_run_all_setups_again(case):
            case.assertEqual(it.upper_run, 3)
            case.assertEqual(it.lower_run, 2)


it.createTests(globals())



import unittest


class TestWithExpectedFailures(unittest.TestCase):
    @unittest.expectedFailure
    def test_should_fail(self):
        assert False

    @unittest.expectedFailure
    def test_should_pass(self):
        assert True

    def test_whatever(self):
        assert True

    def test_fails(self):
        assert False



"""Test doctests plugin."""
import doctest
from textwrap import dedent

from nose2 import events, loader, session
from nose2.plugins import doctests
from nose2.tests._common import TestCase


class UnitTestDocTestLoader(TestCase):

    """Test class DocTestLoader."""

    tags = ["unit"]

    _RUN_IN_TEMP = True

    def setUp(self):
        self.session = session.Session()
        self.loader = loader.PluggableTestLoader(self.session)
        self.plugin = doctests.DocTestLoader(session=self.session)
        super().setUp()

    def test___init__(self):
        """Test the __init__ method."""
        self.assertEqual(self.plugin.extensions, [".txt", ".rst"])

    def test_handle_file(self):
        """Test method handleFile."""
        # Create doctest files of supported types
        doc_test = """\
>>> 2 == 2
True
"""
        txt_event = self._handle_file("docs.txt", doc_test)
        rst_event = self._handle_file("docs.rst", doc_test)
        # Exercise loading of doctests from Python code
        py_event = self._handle_file(
            "docs.py",
            """\
\"\"\"
>>> 2 == 2
True
\"\"\"
""",
        )
        for event, ext in [(txt_event, "txt"), (rst_event, "rst")]:
            (test,) = event.extraTests
            self.assertTrue(isinstance(test, doctest.DocFileCase))
            self.assertEqual(repr(test), "docs.%s" % ext)

        (testsuite,) = py_event.extraTests
        (test,) = list(testsuite)
        self.assertEqual(repr(test), "docs ()")

    def test_handle_file_python_without_doctests(self):
        """Test calling handleFile for a Python module without doctests."""
        event = self._handle_file(
            "mod.py",
            """\
def func():
    pass
""",
        )
        self.assertEqual(event.extraTests, [doctest.DocTestSuite()])

    def test_handle_file_python_setup_py(self):
        # Test calling handleFile on a top-level setup.py file.
        # The file should be ignored by the plugin as it cannot safely be
        # imported.

        setup_py = dedent(
            """\
            '''
            >>> never executed
            '''
            from setuptools import setup
            setup(name='foo')
            """
        )
        event = self._handle_file("setup.py", setup_py)
        self.assertEqual(event.extraTests, [])

    def _handle_file(self, fpath, content):
        """Have plugin handle a file with certain content.

        The file is created, then a plugin is instantiated and its handleFile
        method is called for the file.
        """
        fh = open(fpath, "w")
        try:
            fh.write(content)
        finally:
            fh.close()

        event = events.HandleFileEvent(self.loader, fh.name, fpath, None, None)
        self.plugin.handleFile(event)
        return event
